/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 10:46
  filename: D:\Projects\Apgv5_0\ApgUtilities\ApgUtilities.h
   purpose: Header file for the APG utilities.
  
*********************************************************************/
#ifndef _ApgUtilities_h_16_6_2006_16_21_
#define _ApgUtilities_h_16_6_2006_16_21_

#include "Apg.h"
#include <stdio.h>

// Begin C-language linkage specification
#if defined(__cplusplus)
extern "C"
{
#endif
  
//////////////////////////////////////////////////////////////////////////
// FILE HELPERS
//////////////////////////////////////////////////////////////////////////
ulong ulGetFileSize(const char* cpFileName);
ulong ulGetFile(const char* cpFileName, void* vpBuffer);
ulong ulWriteFile(const char* cpFileName, void* vpData, ulong ulDataLen);
ulong ulRenameFile(const char* cpFileName);

//////////////////////////////////////////////////////////////////////////
// IPv4 & IPv6 HAND-CODED PARSERS
//////////////////////////////////////////////////////////////////////////
#define IPV6_COMPRESS 0xFFFFFFFF
#define IPV6_MAXLEN   45
#define IPV4_MAXLEN   15
#define IPV6_SHORTS   8
#define IPV6_OCTETS   16
#define IPV4_OCTETS   4
#undef  IPV6_COLON
#undef  IPV6_PERIOD
#undef  IPV6_SLASH
#define IPV6_COLON     ':'
#define IPV6_PERIOD    '.'
#define IPV6_SLASH     '/'

// each returns true if conversion was successful
ulong ulIPv4StringToBinary(APG_BSTR* spString, uchar* ucpAddress, ulong* ulpIsMulticast);
ulong ulIPv4BinaryToString(uchar* ucpAddress, char* cpString);
ulong ulIPv6StringToBinary(APG_BSTR* spString, ushort* uspAddress, ulong* ulpIsMulticast);
ulong ulIPv6BinaryToString(ushort* uspAddress, char* cpString);
ulong ulIPv6BinaryToIPv4Form(ushort* uspAddress, char* cpString);
ulong ulExpand(ulong* ulpDigits, ulong ulCount);
ulong ulMaskPrefix(ushort* uspSections, ulong ulPrefixLength);
ulong ulConvertHexString(char* cpInt, ulong ulLen, ulong* ulpValue);
ulong ulConvertToNormalForm(ulong* ulpIPv6, ulong ulCount, ulong ulComp, ulong ulPerLen, uchar* ucpIPv4, ushort* uspIPv6Address);

//////////////////////////////////////////////////////////////////////////
// DISPLAYS
//////////////////////////////////////////////////////////////////////////
// print the alert state
void vPrintAlertState(int iID, unsigned int uiLine, char* cpFile);

// alert handler that prints the alert state and terminates
void vTerminalAlert(int iID, unsigned int uiLine, char* cpFile);

// print file with one-based line numbers
void vDisplaySourceLines(char* cpSrc, FILE* spOut);

// prints the parser state
void vPrintState(const APG_PARSER_STATE* spState, char* cpTitle);

// prints the parse statistics
void vPrintStats(void* vpCtx);

// prints the list of error locations in the error queue
void vPrintErrors(void* vpCtx);

// counts the total number of node visits from statistics
ulong ulNodeVisits(void* vpCtx);

// memory statistics
void vPrintMemStats(void* vpCtx);

// AST
void vDisplayParseTree(void* vpCtx);

// print a Predictive Parsing Table
void vPrintPPT(uchar* ucpPPT, ulong ulCharSetSize);

// print an indent of ulIndent spaces
void vIndent(ulong ulIndent);

// display packrat statistics
void vPrintPackratRuleStats(void* vpCtx);

// display GNU GPL and copyright notice
void vLicenseNotice();

// End C-language linkage specification
#if defined(__cplusplus)
}
#endif

#endif // _ApgUtilities_h_16_6_2006_16_21_
