/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 10:46
  filename: D:\Projects\Apgv5_0\ApgUtilities\Files.c
   purpose: Simple file manipulation functions.
  
*********************************************************************/
#include "ApgUtilities.h"

//////////////////////////////////////////////////////////////////////////
// FILE HELPERS
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Discover and report the size (in bytes) of a file.
//
// FUNCTION: ulong ulGetFileSize
//
//      ARG: 1) const char* cpFileName
//              : the file name
//
//   RETURN: the number of bytes in the file
//           zero if the named file does not exist
//
//////////////////////////////////////////////////////////////////////////
ulong ulGetFileSize(const char* cpFileName)
{
  FILE* spFile = NULL;
  int iTest;
  ulong ulReturn = 0;
  ulong ulLen;
  DASSERT(cpFileName);
  
  while(APG_TRUE)
  {
    spFile = fopen(cpFileName, "rb");
    if(!spFile){break;}
    iTest = fseek(spFile, 0, SEEK_END);
    if(iTest != 0){break;}
    ulLen = ftell(spFile);

    // success
    ulReturn = ulLen;
    break;
  }

  if(spFile){fclose(spFile);}
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Open a file, read the entire file into a buffer
//              and close the file
//
// FUNCTION: ulong ulGetFile
//
//      ARG: 1) const char* cpFileName
//              : the file name
//
//      ARG: 2) void* vpBuffer
//              : pointer to the memory location to copy the file contents to
//                caller must insure that it is big enough to hold the file
//
//   RETURN: the number of bytes copied to the buffer
//
//////////////////////////////////////////////////////////////////////////
ulong ulGetFile(const char* cpFileName, void* vpBuffer)
{
  ulong ulSize;
  FILE* spFile = NULL;
  int iTest;
  ulong ulReturn = 0;
  ulong ulLen;
  DASSERT(cpFileName);
  DASSERT(vpBuffer);
  
  while(APG_TRUE)
  {
    spFile = fopen(cpFileName, "rb");
    if(!spFile){break;}
    iTest = fseek(spFile, 0, SEEK_END);
    if(iTest != 0){break;}
    ulLen = ftell(spFile);
    iTest = fseek(spFile, 0, SEEK_SET);
    if(iTest != 0){break;}
    ulSize = fread(vpBuffer, 1, ulLen, spFile);
    if(ulSize != ulLen){break;}
    
    // success
    ulReturn = ulLen;
    break;
  }
  
  if(spFile){fclose(spFile);}
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Write an array of bytes to a file.
//
// FUNCTION: ulong ulWriteFile
//
//      ARG: 1) const char* cpFileName
//              : the name of the file to use
//                if the file already exists, its data wil be overwritten
//
//      ARG: 2) void* vpData
//              : pointer to the memory location of the first byte to write
//
//      ARG: 3) ulong ulDataLen
//              : number of bytes to write.
//
//   RETURN: number of bytes written
//           if not equal to ulDataLen, an error occurred
//
//////////////////////////////////////////////////////////////////////////
ulong  ulWriteFile(const char* cpFileName, void* vpData, ulong ulDataLen)
{
  ulong ulReturn = 0;
  FILE* spFile = NULL;
  ulong ulBytesWritten = 0;
  DASSERT(cpFileName);

  while(APG_TRUE)
  {
    // validate
    if(!vpData){break;}
    if(ulDataLen ==0){break;}
    
    // open the file
    spFile = fopen(cpFileName, "wb");
    if(!spFile){break;}
    
    // write the data
    ulBytesWritten = fwrite(vpData, 1, ulDataLen, spFile);
    if(ulBytesWritten != ulDataLen){break;}
    
    // success
    ulReturn = ulDataLen;
    break;
  }
  
  if(spFile){fclose(spFile);}
  return ulReturn;
}

/*
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// test to see if the file exists
// if so, back it up with rename filename_date_time
//
// FUNCTION: ulong ulRenameFile
//
//      ARG: 1) const char* cpFileName
//              : file name
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulRenameFile(const char* cpFileName)
{
  bool bReturn = false;
  FILE* spFile = fopen(cpFileName, "r");
  if(spFile)
  {
    // the file exists - rename the file
    fclose(spFile);
    
    // get the date and time
    char caDate[9];
    char caTime[9];
    _strdate(caDate);
    _strtime(caTime);
    caDate[2] = '\0';
    caDate[5] = '\0';
    caTime[2] = '\0';
    caTime[5] = '\0';
    
    // dissect the file name
    char    caNewName[_MAX_PATH];
    char    caDrive[_MAX_DRIVE];
    char    caFile[_MAX_PATH];
    char    caName[_MAX_PATH];
    char    caExt[_MAX_EXT];
    _splitpath( cpFileName, caDrive, caFile, caName, caExt );
    
    strcat(caName, "_");
    strcat(caName, &caDate[0]);
    strcat(caName, &caDate[3]);
    strcat(caName, &caDate[6]);
    strcat(caName, "_");
    strcat(caName, &caTime[0]);
    strcat(caName, &caTime[3]);
    strcat(caName, &caTime[6]);
    
    // create a new backup file name
    _makepath( caNewName, caDrive, caFile, caName, caExt );
    
    // rename the existing file
    rename(cpFileName, caNewName);
    bReturn = APG_TRUE;
  }
  // else // the file does not exists
  return bReturn;
}
*/

