/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 13:06
  filename: D:\Projects\Apgv5_0\Generator\Generator.h
   purpose: Defines all of the Generator's public function declarations.
  
*********************************************************************/
#ifndef _Generator_h_23_9_2006_7_09_
#define _Generator_h_23_9_2006_7_09_

//////////////////////////////////////////////////////////////////////////
// STANDARD INCLUDES
//////////////////////////////////////////////////////////////////////////
#include "Apg.h"
#include "Private.h"
#include "Operators.h"
#include "OperatorsPPT.h"
#include "ApgUtilities.h"
#include "GeneratorDefs.h"

// CommandLine.c
ulong ulCommandLine(CM_CONFIG* spConfig, int argc, char** argv);

// Generator.c
GENCTX* spGenCtor(CM_CONFIG* spConfig);
void    vGenDtor(GENCTX* spData);
void    vGenErrors(GENCTX* spGenCtx);
ulong   ulGenOutput(GENCTX* spGenCtx, char* cpParserName);

// Grammar.c
ulong ulGrammar(GENCTX* spData);
ulong ulGrammarParser(GENCTX* spData, void* vpLexer, void* vpParser);
ulong ulGrammarMetrics(GENCTX* spData);
ulong ulGrammarValidate(GENCTX* spData, APG_BSTR* spGrammar);
void  vGrammarRuleNameControl(GENCTX* spData, APG_BSTR* spSyntax, APG_BSTR* spSemantics);
void  vGrammarFileSections(GENCTX* spData, APG_BSTR* spFile, APG_BSTR* spGrammar, APG_BSTR* spSyntax, APG_BSTR* spSemantics);
ulong ulGrammarRuleLookup(GENCTX* spData, const uchar* ucpSrc, ulong ulSrcLen);
void  vGrammarCharacterSet(GENCTX* spData);

// Erratum.c
void* vpGrammarErrorsCtor(void* vpMemCtx, void* vpLinesCtx);
void  vGrammarErrorsDtor(void* vpErrorsCtx);
ulong ulGrammarErrorsReport(void* vpErrorsCtx, ulong ulLineIndex, char* cpMsg);
ulong ulGrammarErrorsCount(void* vpErrorsCtx);
void  vGrammarErrorsInit(void* vpErrorsCtx);
ulong vGrammarErrorsNext(void* vpErrorsCtx, ulong* ulSrcLineNo, char* cpSrcLine, char* cpErrMsg);

// NonPPTOpcodes.c
ulong ulProductionOpcodes(GENCTX* spData);

// PPTOpcodes.c
ulong ulPPTProductionOpcodes(GENCTX* spData);
ulong ulIsDeterminate(uchar* ucpPPT, ulong ulSize);

// SEST.c
ulong ulSESTMetrics(GENCTX* spData);
ulong ulBranchSESTMetrics(GENCTX* spData);
ulong ulSESTRecursionAnalysis(GENCTX* spGenCtx);

// Display.c
void vPrintLoc(char* cpMsg, unsigned int uiLine, char* cpFile);
void vPrintIntermediateOpcodes(GENCTX* spData);
void vPrintProductionOpcodes(GENCTX* spData);
void vPrintPPTProductionOpcodes(GENCTX* spData);
void vPrintGrammarMetrics(GENCTX* spData);
void vPrintPPTGrammarMetrics(GENCTX* spData);
void vPrintSESTMetrics(GENCTX* spData);
void vPrintTokens(GENCTX* spData);
void vPrintRecursiveAttrs(GENCTX* spData);
void vPrintRuleAttrs(GENCTX* spData);
void vPrintCharMap(GENCTX* spData);

// SESTWalker.c
// typedef for tree node call back function
typedef ulong (*PFN_SESTCB)(ulong ulState, ulong ulTreeDepth, ulong ulChildCount, ulong ulCountMax, OP_INT_ALL* spOp, void* vpData);
void* vpSESTCtor(GENCTX* spData, ulong ulTraversal, void* vpCBData);
void  vSESTDtor(void* vpCtx);
void  vSESTSetCallBacks(void* vpCtx, PFN_SESTCB* ppfnCB);
ulong ulSESTWalk(void* vpCtx, ulong ulStartRule, ulong ulMaxBranches, ulong ulTrace);
ulong ulSESTMaxTreeDepth(void* vpCtx);

#endif // _Generator_h_23_9_2006_7_09_
