/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 13:07
  filename: D:\Projects\Apgv5_0\Generator\GeneratorDefs.h
   purpose: Defines all of the public structs, defines, etc. used
            by the Generator.
  
*********************************************************************/
#ifndef _NewGenerator_h_29_11_2006_11_45_
#define _NewGenerator_h_29_11_2006_11_45_

// runtime libraries
#include <stdio.h>
#include <time.h>
#include <ctype.h>
#include <search.h>

//////////////////////////////////////////////////////////////////////////
// COMMON DEFINES
/////////////////////////////////////////////////////////////////////////
// configuration
#define CM_NAME_MAX 128
typedef struct 
{
  ulong ulError;
  ulong ulLog;
  ulong ulInput;
  ulong ulParserName;
  ulong ulDisplayVerbose;
  ulong ulDisplayGrammar;
  ulong ulDisplayErrors;
  ulong ulDisplayWarnings;
  ulong ulDisplayState;
  ulong ulDisplayTokens;
  ulong ulDisplayOpcodes;
  ulong ulDisplayPPTs;
  ulong ulDisplayMetrics;
  ulong ulDisplayRecursion;
  ulong ulDisplayRuleAttrs;
  ulong ulDisplayCharMap;
  ulong ulNoRecursionAnalysis;
  ulong ulPPT;
  ulong ulMaxTerminalNodes;
  ulong ulStartRule;
  ulong ulCpp;
  char caLog[CM_NAME_MAX];
  char caParserName[CM_NAME_MAX];
  char caInput[CM_NAME_MAX];
} CM_CONFIG;

// custom assertion
#define GENASSERT(cond) if(!(int)(cond)) {ulErrReport(spData->vpMemCtx, APG_GEN_ID, __LINE__, __FILE__); goto APG_EXIT;}

// single-line errors and warnings
#define PUSH_ERROR(cond, msg)
#define PUSH_ERROR_BREAK(cond, msg)
#define PUSH_WARNING(cond, msg)

// tree indentation helpers
#define INDENT(indent) {int test=indent;while(test--)printf(" ");}

// alphabet symbol indexes for SABNF tokens
#define TOK_LINEEND 0
#define TOK_NUM     1
#define TOK_NAMEDEF 2
#define TOK_DEF     3
#define TOK_DEFALT  4
#define TOK_LPAREN  5
#define TOK_RPAREN  6
#define TOK_LOPTION 7
#define TOK_ROPTION 8
#define TOK_MIN     9
#define TOK_MAX     10
#define TOK_RNM     11
#define TOK_ALT     12
#define TOK_CAT     13
#define TOK_REP     14
#define TOK_AND     15
#define TOK_NOT     16
#define TOK_TRG     17
#define TOK_TBS     18
#define TOK_TLS     19
#define TOK_ERROR   20

//////////////////////////////////////////////////////////////////////////
// INTERMEDIATE OPCODES
//////////////////////////////////////////////////////////////////////////
// operator IDs
enum
{
  OP_INT_ID_RNM = 0,
  OP_INT_ID_ALT,
  OP_INT_ID_CAT,
  OP_INT_ID_REP,
  OP_INT_ID_PRD,
  OP_INT_ID_TRG,
  OP_INT_ID_TBS,
  OP_INT_ID_TLS,
  OP_INT_ID_MAX,
};

// node attributes
#define ROOT_RULE 1    // Node or Non-terminal RNM node
#define LEAF_RULE 2    // Leaf or recursive terminal RNM node
typedef struct 
{
  ulong ulEmpty;            // all leaves are empty for at least one derivation
  ulong ulTerminal;         // all leaves are terminal or empty(with at least one terminal) for at least one derivation
  ulong ulFinite;           // accepts finite strings
  ulong ulLeft;             // left recursive
  ulong ulRight;            // right recursive
  ulong ulNested;           // nested recursive
  ulong ulCyclic;           // cyclic
  ulong ulRecursive;        // recursive of any type
  ulong ulRef;              // referenced by the start rule
  ulong ulFirstCat;         // helper for nested recursion detection
  void* vpCatChildStack;    // stack holding vector of CAT child attributes for CAT recursive attribute calculations
} R_ATTR;

typedef struct
{
  R_ATTR sChild;
  R_ATTR sLeftPrefix;
  R_ATTR sRightSuffix;
} CAT_HELPER;

typedef struct
{
  ulong ulIndex;    // index of the PPT in the PPT vector
  ulong ulTouched;  // true if this opcode or rule appears in the production SEST
  ulong ulComplete; // true if the PPT has been completely computed
} PPT_ATTR;

// intermediate opcode
// - single opcode with variables for any of the eight opcodes (trades space for efficiency)
// - production opcodes will be smaller in size and number
// - designed so that all initial values are zero and can be initialized with "memset(0)"
typedef struct  
{
  // production opcode info
  ulong ulOpId;           // operator ID
  ulong ulRuleIndex;      // rule index of this rule opcode (if ulOpId = OP_INT_ID_RNM)
  ulong ulCount;          // number of concatenations/repetitions (if ulOpId = OP_INT_ID_ALT/CAT)
  ulong ulMin;            // minimum number of repetitions(ulOpId = OP_INT_ID_REP/TRG)
  ulong ulMax;            // maximum number of repetitions(ulOpId = OP_INT_ID_REP/TRG)
  ulong ulAnd;            // APG_TRUE if AND predicate, else NOT(if ulOpId = OP_INT_ID_PRD)
  ulong ulOffset;         // offset into string table(ulOpId = OP_INT_ID_TBS/TLS)
  ulong ulLength;         // number characters in the defining string(ulOpId = OP_INT_ID_TBS/TLS)
  ulong ulIndexThis;      // opcode index of this opcode
  ulong ulIndexLast;      // opcode index of last intermediate opcode in the operator

  // grammar info
  ulong ulLineNo;         // grammar file line number (one-based) where this operator was defined
  ulong ulRuleNo;         // rule index of the rule this opcode belongs to
  
  // PPT info
  PPT_ATTR sPPTAttr;      // attributes used for intermediate computation of opcode PPTs
} OP_INT_ALL;

//////////////////////////////////////////////////////////////////////////
// INTERMEDIATE RULE LIST
//////////////////////////////////////////////////////////////////////////
typedef struct  
{
  ulong ulOpcodeOffset;   // index of the first opcode for this rule
  ulong ulOpcodeCount;    // number of opcodes in this rule
  ulong ulNameOffset;     // offset to the first byte of the rule name in the string table
  ulong ulNameLen;        // name length (not including null-term)
  ulong ulLineNo;         // line number of original source where this rule is defined
  ulong ulLeaf;           // APG_TRUE if this rule is a SEST leaf
  R_ATTR sAttr;           // node attributes of this rule
  PPT_ATTR sPPTAttr;      // attributes used for computation of the intermediate rule list
  ulong ulP2Complete;     // true if this rule has been traversed once in phase II
  ushort usSyntax;         // true for call to call back function during syntax analysis
  ushort usSemantics;      // true for call to call back function during semantic analysis
  uchar ucReserved1;      // not used
  uchar ucReserved2;      // not used
} INT_RULE_LIST;

//////////////////////////////////////////////////////////////////////////
// PARSING DATA
//////////////////////////////////////////////////////////////////////////
// SABNF token
typedef struct  
{
  ulong ulID;
  ulong ulOffset;
  ulong ulLen;
  ulong ulLineNo;
  ulong ulMin;
  ulong ulMax;
  ulong ulRuleIndex;
} SABNFTOKEN;

typedef struct 
{
  // rules
  ulong ulRuleCount;    // number of rule names in the grammar

  // character set
  ulong ulCharSetSize;  // number of characters in the symbol table
  ulong ulCharSetUsed;  // number of characters actually used by TRG, TBS and TLS

  // opcodes
  ulong ulALT;          // number of production alternations
  ulong ulAlternates;   // the total number of alternates (ALT children)
  ulong ulCAT;          // number of production concatenations
  ulong ulConcats;      // the total number of CAT children
  ulong ulREP;          // number of production repetitions
  ulong ulPRD;          // number of production syntactic predicates
  ulong ulRNM;          // number of production rule name operators
  ulong ulTRG;          // number of production terminal range
  ulong ulTBS;          // number of production terminal binary string
  ulong ulTLS;          // number of production terminal literal string
} GRAMMAR_METRICS;

#define SEST_RECURSION  0
#define SEST_PPT        1
typedef struct 
{
  // total tree nodes (interior tree branch nodes + tree branch leaves)
  ulong ulTotalNodes;
  ulong ulMaxTreeDepth;

  // non-terminal nodes (interior tree branch nodes)
  ulong ulTotalNonTerminal;
  ulong ulALTNodes;
  ulong ulCATNodes;
  ulong ulREPNodes;
  ulong ulPRDNodes;
  ulong ulRNMNodes;

  // terminal nodes (tree branch leaves)
  ulong ulTotalTerminal;      // RNM(leaf) + TRG + TBS + TLS leaves
  ulong ulTerminalLeaves;     // TRG + TBS + TLS leaves
  ulong ulRNMLeaves;          // once-expanded recursive rule leaves
  ulong ulRootRNMLeaves;      // once-expanded root recursive rule leaves
  ulong ulStartRule;           // redundant, but convenient for gathering SEST metrics
  ulong ulEmptyStringLeaves;
  ulong ulTRGLeaves;
  ulong ulTBSLeaves;
  ulong ulTLSLeaves;

  // recursive rule locater helper
  INT_RULE_LIST* spRules;
} SEST_METRICS;

typedef struct 
{
  // executable parser data (all data in 8-bit bytes)
  ulong ulTotalExeData;
  ulong ulStringTable;
  ulong ulRuleList;
  ulong ulOpcodes;
  ulong ulPPT;
} EXECUTABLE_METRICS;

// PPT-dependent function prototypes
// typedef ulong (*PFN_PRODOPCODES)(void* vpCtx, void* vpRulesCtx, void* vpOpcodeCtx);
// typedef ulong (*PFN_PRODRULES)(void* vpCtx, INT_RULE_LIST* spIntRules, ulong ulRuleCount, void* vpRulesCtx);

// global data to span main(), both parsers and SEST analysis
typedef struct  
{
  // basic components
  void*   vpValidate;     // must be equal to the context address
  void*   vpMemCtx;          // memory component handle
  void*   vpLinesCtx;     // vector of input string lines & characters
  void*   vpErrorsCtx;   // grammar errors reporting component
  void*   vpWarningsCtx;  // grammar warnings reporting component

  // grammar info
  APG_BSTR  sSource;
  APG_BSTR  sGrammar;
  APG_BSTR  sSyntaxControl;
  APG_BSTR  sSemanticsControl;
  CM_CONFIG sConfig;
  
  // string table
  void*   vpStrTblCtx;    // string table to support opcodes

  // rules
  void*   vpIntRuleCtx;   // vector of intermediate rules
  void*   vpRuleCtx;      // vector of production rules
  ulong   ulRuleCount;    // number of rules in the rule list
  ulong   ulStartRule;    // the root rule - defines the grammar
  ulong   ulIntRuleCount; // intermediate rule count
  
  // opcodes
  void*   vpIntOpcodesCtx;// vector of intermediate opcodes
  void*   vpOpcodeCtx;    // vector of production opcodes
  ulong   ulOpcodeCount;  // number of production opcodes
  ulong   ulIntOpcodeCount; // number of intermediate opcodes
  
  // character set info - NOTE: character means zero-based index of an alphabet symbol
  ulong   ulCharSetSize;  // number of characters defined by terminal operators
  ulong   ulCharSetUsed;  // the number of characters actually used by terminal operators
  ulong   ulCharSetUnused;// (redundant = ulCharSetSize - ulCharSetUsed)
  ulong   ulEOFSetSize;   // (redundant = ulCharSetSize + 1) number of characters including EOF
  ulong   ulMaxChar;      // (redundant = ulCharSetSize - 1) the last character in the alphabet
  ulong   ulEOFChar;      // (redundant = ulCharSetSize) used as EOF character(zero-based index) in PPT

  // token tables
  void*   vpTokTblCtx;    // SABNF token table
  void*   vpTokIdxCtx;    // token index table


  // NOTE: POSSIBLE FUTURE USE
  // mapping of external character set (with possibly unused characters)
  // to the internal character set (compressed to only the used characters)
  void*   vpMapXCtx;      // maps external to internal character set
  void*   vpMapICtx;      // maps internal to external character set

  // generated parser info
  void*   vpTransNamesCtx;// list of null-term call back function names
  void*   vpIDNamesCtx;   // list of null-term rule name IDs
  
  // PPT info
  ulong   ulPPT;          // true if PPTs are used, false otherwise
  void*   vpIntPPTCtx;    // vector of intermediate PPTs
  void*   vpPPTCtx;       // vector of production PPTs
  ulong   ulPPTSize1;     // size, in bytes, of a 1-bit PPT
  ulong   ulPPTSize2;     // size, in bytes, of a 2-bit PPT
  
  // temporary parsing/working data
  uchar*  ucpSrc;         // original ASCII ABNF grammar file (not necessarily the same as sSource.ucpSrc)
  ulong   ulSrcLen;       // number of characters in the original grammar file
  ulong   ulLineNo;       // carries the current original grammar file line number
  ulong   ulRuleNo;       // carries rule index (SABNFParser)
  void*   vpAltStackCtx;  // stack of alternates for the current rule name
  void*   vpCatStackCtx;  // stack of concatenations for the current rule name
  void*   vpRepStackCtx;  // stack of repetitions for the current rule name
  SABNFTOKEN  sToken;     // for building tokens across several rules (Lexer)

  // statistics
  R_ATTR              sGrammarAttrs;
  GRAMMAR_METRICS     sGrammarMetrics;
  SEST_METRICS        sSESTMetrics;
  EXECUTABLE_METRICS  sExeMetrics;
} GENCTX;

#endif // _NewGenerator_h_29_11_2006_11_45_
