/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 17:47
  filename: D:\Projects\Apgv5_0\Samples\IPv6\LookAhead.c
   purpose: Testing of efficiency improvements using
                Semantic Look Ahead.
  
*********************************************************************/
#include "main.h"

static void    vRunLookAhead(TESTDATA* spData);
void vTestLookAhead(void* vpMemCtx, char* cpFileName)
{
  APG_BSTR          sSrc;
  void*             vpBstr;
  ulong             ulCount;
  double            dPerCent;
  TESTDATA          sData;
  APG_PARSER_STATS  sStats;
  APG_PARSER_STATS  sStatsLookAhead;

  sSrc.ulLen = ulGetFileSize(cpFileName);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  MAINASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpFileName, (void*)sSrc.ucpStr);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  MAINASSERT(vpBstr);
  ulCount = ulReadAddresses(&sSrc, vpBstr);
  MAINASSERT(ulCount);
 
  // common input parameters
  sData.vpMemCtx = vpMemCtx;
  sData.vpMsgs = vpBstr;
  sData.ulRepetitions = 1;

  // run without look ahead
  sData.vpInit = vpIPv6SyntaxOnlyInit;
  sData.cpTitle = NULL;
  vRunLookAhead(&sData);
  sStats = sData.sStats;

  if(sStats.ulNodesMaxDepth == 0)
  {
    printf("\n");
    printf("//////////////////////////////////////////////////////////////////////////\n");
    printf("// LOOK AHEAD SAVINGS\n");
    printf("//////////////////////////////////////////////////////////////////////////\n");
    printf("(requires Apg.lib statistics)\n");
    printf("(_APG_CFG_DEBUG or _APG_CFG_STATS must be defined)\n");
    return;
  } 
  
  // run with look ahead without PPT
  sData.vpInit = vpIPv6LookAheadInit;
  sData.cpTitle =NULL;
  vRunLookAhead(&sData);
  sStatsLookAhead = sData.sStats;
  
  // print cumulative statistics
  printf("\n");
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("// LOOK AHEAD SAVINGS (without PPT) (for %lu Addresses)\n", ulCount);
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("backtracks\n");
  printf(" - without look ahead = %lu\n", sStats.ulBackTrackCount);
  printf(" -    with look ahead = %lu\n", sStatsLookAhead.ulBackTrackCount);
  dPerCent = (double)((sStats.ulBackTrackCount - sStatsLookAhead.ulBackTrackCount) * 100.0) / (double)sStats.ulBackTrackCount;
  printf("              %% saved = %1.3g\n", dPerCent);

  printf("\n");
  printf("backtracked characters\n");
  printf(" - without look ahead = %lu\n", sStats.ulBackTrackChars);
  printf(" -    with look ahead = %lu\n", sStatsLookAhead.ulBackTrackChars);
  dPerCent = (double)((sStats.ulBackTrackChars - sStatsLookAhead.ulBackTrackChars) * 100.0) / (double)sStats.ulBackTrackChars;
  printf("              %% saved = %1.3g\n", dPerCent);

  // no savings with PPT
  // run with look ahead with PPT
//   sData.vpInit = vpIPv6LookAheadPPTInit;
//   sData.cpTitle =NULL;
//   vRunLookAhead(&sData);
//   sStatsLookAhead = sData.sStats;
//   
//   // print cumulative statistics
//   printf("\n");
//   printf("//////////////////////////////////////////////////////////////////////////\n");
//   printf("// LOOK AHEAD SAVINGS (with PPT) (for %lu Addresses)\n", ulCount);
//   printf("//////////////////////////////////////////////////////////////////////////\n");
//   printf("backtracks\n");
//   printf(" - without look ahead = %lu\n", sStats.ulBackTrackCount);
//   printf(" -    with look ahead = %lu\n", sStatsLookAhead.ulBackTrackCount);
//   dPerCent = (double)((sStats.ulBackTrackCount - sStatsLookAhead.ulBackTrackCount) * 100.0) / (double)sStats.ulBackTrackCount;
//   printf("              %% saved = %1.3g\n", dPerCent);
//   
//   printf("\n");
//   printf("backtracked characters\n");
//   printf(" - without look ahead = %lu\n", sStats.ulBackTrackChars);
//   printf(" -    with look ahead = %lu\n", sStatsLookAhead.ulBackTrackChars);
//   dPerCent = (double)((sStats.ulBackTrackChars - sStatsLookAhead.ulBackTrackChars) * 100.0) / (double)sStats.ulBackTrackChars;
//   printf("              %% saved = %1.3g\n", dPerCent);
  
  // clean up resources
  vVecDtor(vpBstr);
  vMemFree(vpMemCtx, sSrc.ucpStr);
}

void vRunLookAhead(TESTDATA* spTestData)
{
  APG_PARSER_STATE  sState;
  APG_PARSER_STATS  sStats;
  APG_BSTR*         spBstr;
  ulong             ulTest;
  ulong             ulCount;
  ulong             ulAdd;
  ulong*            ulpStats;
  ulong*            ulpTotalStats;
  void*             vpParserCtx;
  DASSERT(spTestData);
  DASSERT(spTestData->vpMemCtx);
  DASSERT(spTestData->vpMsgs);
  DASSERT(spTestData->vpInit);
  
  memset((void*)&spTestData->sStats, 0, sizeof(spTestData->sStats));
  if(spTestData->cpTitle && *spTestData->cpTitle != 0)
  {
    // print a header
    printf("\n");
    printf("*** %s\n", spTestData->cpTitle);
  }
  
  // get the list of addresses
  spBstr = (APG_BSTR*)vpVecFront(spTestData->vpMsgs);
  MAINASSERT(spBstr);
  ulCount = ulVecSize(spTestData->vpMsgs);
  MAINASSERT(ulCount);
  
  // initialize the parser context
  ulTest = ulParserInit(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(ulTest);
  vpParserCtx = vpParserCtor(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(vpParserCtx);
  
  // translate each address
  for(ulAdd = 0; ulAdd < ulCount; ++ulAdd, ++spBstr)
  {
    ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, spBstr->ucpStr, spBstr->ulLen, spBstr, APG_FALSE);
    MAINASSERT(ulTest);
    ulTest = ulParserState(vpParserCtx, &sState);
    MAINASSERT(ulTest && sState.ulState == MATCH);
    ulTest = ulParserStats(vpParserCtx, &sStats);
    MAINASSERT(ulTest);

    // collective statistics
    ulpTotalStats = (ulong*)&spTestData->sStats;
    ulpStats = (ulong*)&sStats;
    for(ulTest = 0; ulTest < sizeof(sStats)/sizeof(ulong); ++ulTest)
    {
      ulpTotalStats[ulTest] += ulpStats[ulTest];
    }
  }

  // clean up
  vParserDtor(vpParserCtx);
}
