/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:13
  filename: D:\Projects\Apgv5_0\Samples\SDP\Announcements.c
  purpose: Reads a text file of concatenated SDP announcements
  and generates a vector of binary strings, one for
  each announcement.
  
*********************************************************************/
#include "main.h"


#define CR 13
#define LF 10
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// separate a file of announcements into a list of announcements
// announcements are separated by comments
// comments begin at char 0 of a line with ";"
// NOTE: lines must end with CRLF
// NOTE: last announcement MUST be followed by comment
//
// FUNCTION: ulong ulReadAnnouncements
//
//      ARG: 1) APG_BSTR* spSrc
//              : binary string defining the list of announcements
//
//      ARG: 2) void* vpBstr
//              : handle to vector of APG_BSTR structs
//
//   RETURN: returns the number of announcements found
//
//////////////////////////////////////////////////////////////////////////
ulong ulReadAnnouncements(APG_BSTR* spSrc, void* vpBstr)
{
  ulong ulCount = 0;
  ulong ulIndex;
  char* cpAnnBeg;
  char* cpFileEnd;
  char* cpChar;
  APG_BSTR* spBstr;
  
  // single-point-of-exit loop
  cpChar = (char*)spSrc->ucpStr;
  cpFileEnd = cpChar + spSrc->ulLen;
  while(APG_TRUE)
  {
    int i=0;
    // skip comments, if any
    // single-point-of-exit loop
    while(APG_TRUE)
    {
      if(cpChar >= cpFileEnd){goto APG_EXIT;}
      if(*cpChar == ';' || *cpChar == 0)
      {
        // skip to end of line or EOF
        while(*cpChar != CR && cpChar < cpFileEnd){++cpChar;}
        cpChar += 2;
      }
      else
      {
        // get the next announcement
        break;
      }
    } // end of single-point-of-exit loop
    
    // get the next announcement
    // single-point-of-exit loop
    cpAnnBeg = cpChar;
    while(APG_TRUE)
    {
      if(cpChar >= cpFileEnd){goto APG_EXIT;}
      if(!(*cpChar == ';' || *cpChar == 0))
      {
        // skip to end of line or EOF
        while(*cpChar != CR && cpChar < cpFileEnd){++cpChar;}
        cpChar += 2;
      }
      else
      {
        // get the next announcement
        spBstr = (APG_BSTR*)vpVecPush(vpBstr, NULL);
        spBstr->ucpStr = (uchar*)cpAnnBeg;
        spBstr->ulLen = (ulong)(cpChar - cpAnnBeg);
        ++ulCount;
        break;
      }
    } // end of single-point-of-exit loop
  } // end of single-point-of-exit loop
  
APG_EXIT:
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex)
  {
    // put a null term after each message
    // required by vDisplaySourceLines() to display lines
    spBstr = (APG_BSTR*)vpVecAt(vpBstr, ulIndex);
    spBstr->ucpStr[spBstr->ulLen] = 0;  
  }
  
  return ulCount;
}
