/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:16
  filename: D:\Projects\Apgv5_0\Samples\SDP\Origin.c
   purpose: Illustrates lazy parsing. Uses APG parsing of sub-rules
                (rules other than the start rule of the announcement)
                to parse one and only one line within the message.

                Identifies and prints each component of the origin-field.
  
*********************************************************************/
#include "main.h"

static ulong ulPrintOriginField(void* vpCatalogCtx);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Driver for the tests
//
// FUNCTION: ulong ulOriginTest
//
//      ARG: 1) APG_BSTR* spAnnouncement
//              : binary string defining the list of announcements
//
//      ARG: 2) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 3) void* vpParserCtx
//              : pointer to a valid parser component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulOriginTest(APG_BSTR* spAnnouncement, void* vpMemCtx, void* vpParserCtx)
{
  ulong   ulReturn = APG_FALSE;
  APG_PARSER_STATE sState;
  LINE_ITERATOR sIter;
  ulong ulTest;
  APG_BSTR sLine;
  ulong ulID;
  void* vpCatalogCtx;
  ulong ulPhraseCount;

  // iterate over all announcement lines
  ulTest = ulLinesInit(&sIter, spAnnouncement);
  SDPASSERT(ulTest);
  while(APG_TRUE)
  {
    ulTest = ulLinesNext(&sIter, &sLine, &ulID);
    SDPASSERT(ulTest);
    if(sLine.ucpStr == NULL)
    {
      // end of announcement
      break;
    }
    
    if(ulID == SESSION_ORIGIN)
    {
      // parse the origin line
      ulTest = ulParserSyntaxAnalysis(vpParserCtx, RULE_SDP_ORIGIN_FIELD,sLine.ucpStr, sLine.ulLen, NULL, APG_FALSE);
      SDPASSERT(ulTest);
      ulTest = ulParserState(vpParserCtx, &sState);
      SDPASSERT(ulTest);
      if(sState.ulState == MATCH)
      {
        vpCatalogCtx = vpCatalogCtor(vpParserCtx);
        SDPASSERT(vpCatalogCtx);
        ulPhraseCount = ulCatalog(vpCatalogCtx);
        SDPASSERT(ulPhraseCount);
        ulTest = ulPrintOriginField(vpCatalogCtx);
        SDPASSERT(ulTest);
      }
        
      break;
    }
  }
  
  // success
  ulReturn = APG_TRUE;
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Find an dprint all origin fields.
//
// FUNCTION: static ulong ulPrintOriginField
//
//      ARG: 1) void* vpCatalogCtx
//              : pointer to a valid catalog component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulPrintOriginField(void* vpCatalogCtx)
{
  ulong   ulReturn = APG_FALSE;
  char    caBuf[BUFSIZE];
  APG_CAT_ITERATOR sPhraseIter;
  APG_CAT_ITERATOR sChildIter;
  APG_CAT_ITERATOR* spPhraseIter;
  APG_CAT_ITERATOR* spChildIter;
  
  // get the list of origin-fields (should be one)
  spPhraseIter = spCatalogGetPhraseList(vpCatalogCtx, RULE_SDP_ORIGIN_FIELD, &sPhraseIter);
  SDPASSERT(spPhraseIter);
  SDPASSERT(spPhraseIter->ulCount == 1);
  
  printf("\n");
  printf("[%s]\n", spPhraseIter->cpPhraseName);
  SDPASSERT(spPhraseIter->sPhrase.ulLen < BUFSIZE);
  ulCatalogGetString(&spPhraseIter->sPhrase, caBuf);
  printf("%s\n", caBuf);
  
  // get the list of origin-field 1st-level children
  spChildIter = spCatalogGetChildList(vpCatalogCtx, spPhraseIter, &sChildIter);
  while(spChildIter)
  {
    SDPASSERT(spChildIter->sPhrase.ulLen < BUFSIZE);
    ulCatalogGetString(&spChildIter->sPhrase, caBuf);
    printf("\n");
    printf("  [%s]\n", spChildIter->cpPhraseName);
    printf("  %s\n", caBuf);
    spChildIter = spCatalogIterNext(vpCatalogCtx, spChildIter);
  }
  
  // success
  ulReturn = APG_TRUE;
  return ulReturn;
}
