/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:47:25 2007
	filename: SDP.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component SDP.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include "ApgUtilities.h"
//}}Include.pre

#include "SDP.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 11992 (bytes)
//                   = 1.17E+01 (KB)
//                   = 1.14E-02 (MB)
//   character count = 256 (grammar alphabet character set)
// unused characters = 0 (grammar alphabet character set)
//        rule count = 110
//      opcode count = 632
//         ALT count = 43(141) (alternations)
//         CAT count = 96(286) (concatenations)
//         RNM count = 255
//         REP count = 95
//         PRD count = 0
//         TRG count = 51
//         TBS count = 22
//         TLS count = 70
//      string table = 1292 (bytes)
//           opcodes = 7980 (bytes)
//         rule list = 2200 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_announcement(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_session_section(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_proto_version(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_origin_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_session_name_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_information_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_uri_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_email_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_phone_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_connection_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bandwidth_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_time_fields(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_key_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_attribute_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_media_section(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_media_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_media(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_fmt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_proto(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_MediaPort(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_PortCount(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_username(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_sess_id(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_sess_version(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_nettype(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_addrtype(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_FQDN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_extn_addr(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv4Type(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_AddressCount(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv6Type(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv4Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv4Unicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv6Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv6Unicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 36;
static PFN_CALLBACK g_pfnCallbacks[36] = 
{
  pfn_announcement,
  pfn_session_section,
  pfn_proto_version,
  pfn_origin_field,
  pfn_session_name_field,
  pfn_information_field,
  pfn_uri_field,
  pfn_email_field,
  pfn_phone_field,
  pfn_connection_field,
  pfn_bandwidth_field,
  pfn_time_fields,
  pfn_key_field,
  pfn_attribute_field,
  pfn_media_section,
  pfn_media_field,
  pfn_media,
  pfn_fmt,
  pfn_proto,
  pfn_MediaPort,
  pfn_PortCount,
  pfn_username,
  pfn_sess_id,
  pfn_sess_version,
  pfn_nettype,
  pfn_addrtype,
  pfn_FQDN,
  pfn_extn_addr,
  pfn_IPv4Type,
  pfn_AddressCount,
  pfn_IPv6Type,
  pfn_URI_reference,
  pfn_IPv4Multicast,
  pfn_IPv4Unicast,
  pfn_IPv6Multicast,
  pfn_IPv6Unicast,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[2888] = 
{
  0x00002D20,0x769D350C,0x00000000,0x0000006E,0x00000278,0x00000100,0x00000000,0x00000100,
  0x000000FF,0x00000000,0x00000000,0x00000050,0x00000050,0x0000050C,0x0000055C,0x00001F2C,
  0x00002488,0x00000898,0x00000000,0x00000000,0x6F6E6E61,0x65636E75,0x746E656D,0x73657300,
  0x6E6F6973,0x6365732D,0x6E6F6974,0x6F3D7600,0x693D733D,0x653D753D,0x633D703D,0x743D623D,
  0x613D6B3D,0x6F72703D,0x762D6F74,0x69737265,0x6F006E6F,0x69676972,0x69662D6E,0x00646C65,
  0x73736573,0x2D6E6F69,0x656D616E,0x6569662D,0x6900646C,0x726F666E,0x6974616D,0x662D6E6F,
  0x646C6569,0x69727500,0x6569662D,0x6500646C,0x6C69616D,0x6569662D,0x7000646C,0x656E6F68,
  0x6569662D,0x6300646C,0x656E6E6F,0x6F697463,0x69662D6E,0x00646C65,0x646E6162,0x74646977,
  0x69662D68,0x00646C65,0x6D69743A,0x69662D65,0x73646C65,0x723D3D00,0x61657065,0x69662D74,
  0x73646C65,0x6E6F7A00,0x64612D65,0x7473756A,0x746E656D,0x2D2D0073,0x2D79656B,0x6C656966,
  0x74610064,0x62697274,0x2D657475,0x6C656966,0x656D0064,0x2D616964,0x74636573,0x006E6F69,
  0x3D693D6D,0x3D623D63,0x3D613D6B,0x6964656D,0x69662D61,0x00646C65,0x64656D2F,0x66006169,
  0x7000746D,0x6F746F72,0x654D2F00,0x50616964,0x0074726F,0x74726F50,0x6E756F43,0x73750074,
  0x616E7265,0x7300656D,0x2D737365,0x73006469,0x2D737365,0x73726576,0x006E6F69,0x7474656E,
  0x00657079,0x72646461,0x65707974,0x616D6500,0x612D6C69,0x65726464,0x61007373,0x65726464,
  0x612D7373,0x632D646E,0x656D6D6F,0x2800746E,0x73696429,0x6D616E70,0x6E612D65,0x64612D64,
  0x73657264,0x3E3C0073,0x6E6F6870,0x756E2D65,0x7265626D,0x3C292800,0x6F68703E,0x2B00656E,
  0x7477622D,0x00657079,0x646E6162,0x74646977,0x74730068,0x2D747261,0x656D6974,0x74733000,
  0x742D706F,0x00656D69,0x6D697430,0x65720065,0x74616570,0x746E692D,0x61767265,0x7974006C,
  0x2D646570,0x656D6974,0x78696600,0x6C2D6465,0x742D6E65,0x2D656D69,0x74696E75,0x79656B00,
  0x7079742D,0x72700065,0x74706D6F,0x61656C63,0x61623A72,0x34366573,0x6972753A,0x7361623A,
  0x00343665,0x65736162,0x752D3436,0x0074696E,0x65736162,0x702D3436,0x3D006461,0x61623D3D,
  0x34366573,0x6168632D,0x2F2B0072,0x72747461,0x74756269,0x613A0065,0x662D7474,0x646C6569,
  0x74746100,0x6C61762D,0x63006575,0x656E6E6F,0x6F697463,0x79742D6E,0x61006570,0x632D796E,
  0x656E6E6F,0x6F697463,0x5146006E,0x2D004E44,0x7478652E,0x64612D6E,0x49007264,0x43347650,
  0x656E6E6F,0x6F697463,0x5049006E,0x79543476,0x69006570,0x50493470,0x754D3476,0x4369746C,
  0x2F006E6F,0x6C74742F,0x64413000,0x73657264,0x756F4373,0x4900746E,0x43367650,0x656E6E6F,
  0x6F697463,0x5049006E,0x79543676,0x69006570,0x50493670,0x754D3676,0x6F43746C,0x742F006E,
  0x00747865,0x65747962,0x7274732D,0x00676E69,0x2D6E6F6E,0x732D7377,0x6E697274,0x6F740067,
  0x2D6E656B,0x72616863,0x6B6F7400,0x65006E65,0x6C69616D,0x6661732D,0x6C610065,0x2D616870,
  0x656D756E,0x00636972,0x2D495255,0x65666572,0x636E6572,0x65640065,0x636F2D63,0x00746574,
  0x34765049,0x746C754D,0x73616369,0x492E0074,0x55347650,0x6163696E,0x2E007473,0x32544E49,
  0x32003432,0x49333232,0x4D367650,0x69746C75,0x74736163,0x3A666600,0x7650492E,0x696E5536,
  0x74736163,0x612E3A00,0x2D726464,0x63657073,0x6F6C4000,0x2D6C6163,0x74726170,0x6D6F6400,
  0x006E6961,0x616D6F64,0x6C2D6E69,0x72657469,0x5B006C61,0x6F63645D,0x6E65746E,0x74640074,
  0x00747865,0x572D4F4E,0x54432D53,0x7461004C,0x00747865,0x25242321,0x2B2A2726,0x3F3D2F2D,
  0x7B605F5E,0x617E7D7C,0x006D6F74,0x2D746F64,0x6D6F7461,0x746F6400,0x6F74612D,0x65742D6D,
  0x2E007478,0x78657471,0x63710074,0x65746E6F,0x7100746E,0x65746F75,0x74732D64,0x676E6972,
  0x6F757100,0x2D646574,0x72696170,0x626F5C00,0x70712D73,0x626F5C00,0x6F6C2D73,0x2D6C6163,
  0x74726170,0x626F2E00,0x6F642D73,0x6E69616D,0x6F772E00,0x46006472,0x63005357,0x74786574,
  0x6F636300,0x6E65746E,0x6F630074,0x6E656D6D,0x29280074,0x53574643,0x73626F00,0x5357462D,
  0x00505300,0x42415448,0x55514400,0x0045544F,0x464C5243,0x504C4100,0x44004148,0x54494749,
  0x48435600,0x57005241,0x48005053,0x49445845,0x4F500047,0x49442D53,0x00544947,0x00000002,
  0x00000030,0x00000000,0x00000014,0x00000001,0x00000003,0x00000030,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000030,0x00000010,0x00000002,0x00000308,0x00000002,0x00000068,0x00000006,
  0x00000050,0x0000001D,0x00000002,0x00000000,0x0000005C,0x00000002,0x00000000,0x00000068,
  0x00000067,0x00000002,0x00000098,0x00000006,0x00000080,0x0000001F,0x00000002,0x00000000,
  0x0000008C,0x00000003,0x00000000,0x00000098,0x00000067,0x00000002,0x000000C8,0x00000006,
  0x000000B0,0x00000021,0x00000002,0x00000000,0x000000BC,0x00000004,0x00000000,0x000000C8,
  0x00000067,0x00000003,0x00000108,0x00000000,0x00000001,0x00000002,0x00000108,0x00000006,
  0x000000F0,0x00000023,0x00000002,0x00000000,0x000000FC,0x00000005,0x00000000,0x00000108,
  0x00000067,0x00000003,0x00000148,0x00000000,0x00000001,0x00000002,0x00000148,0x00000006,
  0x00000130,0x00000025,0x00000002,0x00000000,0x0000013C,0x00000006,0x00000000,0x00000148,
  0x00000067,0x00000003,0x00000188,0x00000000,0xFFFFFFFF,0x00000002,0x00000188,0x00000006,
  0x00000170,0x00000027,0x00000002,0x00000000,0x0000017C,0x00000007,0x00000000,0x00000188,
  0x00000067,0x00000003,0x000001C8,0x00000000,0xFFFFFFFF,0x00000002,0x000001C8,0x00000006,
  0x000001B0,0x00000029,0x00000002,0x00000000,0x000001BC,0x00000008,0x00000000,0x000001C8,
  0x00000067,0x00000003,0x00000208,0x00000000,0x00000001,0x00000002,0x00000208,0x00000006,
  0x000001F0,0x0000002B,0x00000002,0x00000000,0x000001FC,0x00000009,0x00000000,0x00000208,
  0x00000067,0x00000003,0x00000248,0x00000000,0xFFFFFFFF,0x00000002,0x00000248,0x00000006,
  0x00000230,0x0000002D,0x00000002,0x00000000,0x0000023C,0x0000000A,0x00000000,0x00000248,
  0x00000067,0x00000003,0x00000288,0x00000001,0xFFFFFFFF,0x00000002,0x00000288,0x00000006,
  0x00000270,0x0000002F,0x00000002,0x00000000,0x0000027C,0x0000000B,0x00000000,0x00000288,
  0x00000067,0x00000003,0x000002C8,0x00000000,0x00000001,0x00000002,0x000002C8,0x00000006,
  0x000002B0,0x00000031,0x00000002,0x00000000,0x000002BC,0x0000000E,0x00000000,0x000002C8,
  0x00000067,0x00000003,0x00000308,0x00000000,0xFFFFFFFF,0x00000002,0x00000308,0x00000006,
  0x000002F0,0x00000033,0x00000002,0x00000000,0x000002FC,0x0000000F,0x00000000,0x00000308,
  0x00000067,0x00000003,0x00000324,0x00000001,0xFFFFFFFF,0x00000000,0x00000324,0x00000069,
  0x00000002,0x0000042C,0x00000000,0x00000338,0x00000017,0x00000000,0x00000344,0x00000064,
  0x00000000,0x00000350,0x00000018,0x00000000,0x0000035C,0x00000064,0x00000000,0x00000368,
  0x00000019,0x00000000,0x00000374,0x00000064,0x00000000,0x00000380,0x0000001A,0x00000000,
  0x0000038C,0x00000064,0x00000001,0x0000042C,0x00000002,0x000003C0,0x00000000,0x000003A8,
  0x00000036,0x00000000,0x000003B4,0x00000064,0x00000000,0x000003C0,0x00000047,0x00000002,
  0x000003EC,0x00000000,0x000003D4,0x0000003B,0x00000000,0x000003E0,0x00000064,0x00000000,
  0x000003EC,0x0000004A,0x00000002,0x0000042C,0x00000000,0x00000400,0x0000001B,0x00000000,
  0x0000040C,0x00000064,0x00000001,0x0000042C,0x00000000,0x00000420,0x00000033,0x00000000,
  0x0000042C,0x00000034,0x00000000,0x00000438,0x0000003D,0x00000000,0x00000444,0x0000003D,
  0x00000000,0x00000450,0x00000044,0x00000000,0x0000045C,0x0000001C,0x00000000,0x00000468,
  0x0000001F,0x00000002,0x00000494,0x00000000,0x0000047C,0x0000001A,0x00000000,0x00000488,
  0x00000064,0x00000000,0x00000494,0x00000031,0x00000002,0x000004C4,0x00000000,0x000004A8,
  0x00000021,0x00000007,0x000004B8,0x000000B8,0x00000001,0x00000000,0x000004C4,0x00000022,
  0x00000002,0x00000578,0x00000000,0x000004D8,0x00000023,0x00000000,0x000004E4,0x00000064,
  0x00000000,0x000004F0,0x00000024,0x00000003,0x00000534,0x00000000,0xFFFFFFFF,0x00000002,
  0x00000534,0x00000005,0x00000518,0x00000072,0x00000072,0x00000007,0x00000528,0x000000C5,
  0x00000001,0x00000000,0x00000534,0x0000000C,0x00000003,0x00000578,0x00000000,0x00000001,
  0x00000002,0x00000578,0x00000005,0x0000055C,0x0000007A,0x0000007A,0x00000007,0x0000056C,
  0x000000C6,0x00000001,0x00000000,0x00000578,0x0000000D,0x00000002,0x000005D4,0x00000000,
  0x0000058C,0x00000026,0x00000000,0x00000598,0x00000064,0x00000000,0x000005A4,0x00000027,
  0x00000003,0x000005D4,0x00000001,0xFFFFFFFF,0x00000002,0x000005D4,0x00000000,0x000005C8,
  0x00000064,0x00000000,0x000005D4,0x00000027,0x00000002,0x00000688,0x00000000,0x000005E8,
  0x00000025,0x00000000,0x000005F4,0x00000064,0x00000003,0x00000614,0x00000000,0x00000001,
  0x00000007,0x00000614,0x000000E6,0x00000001,0x00000000,0x00000620,0x00000027,0x00000003,
  0x00000688,0x00000000,0xFFFFFFFF,0x00000002,0x00000688,0x00000000,0x00000644,0x00000064,
  0x00000000,0x00000650,0x00000025,0x00000000,0x0000065C,0x00000064,0x00000003,0x0000067C,
  0x00000000,0x00000001,0x00000007,0x0000067C,0x000000E7,0x00000001,0x00000000,0x00000688,
  0x00000027,0x00000000,0x00000694,0x00000029,0x00000000,0x000006A0,0x0000002E,0x00000002,
  0x00000818,0x00000002,0x000006D8,0x00000006,0x000006C0,0x00000110,0x00000002,0x00000000,
  0x000006CC,0x00000011,0x00000000,0x000006D8,0x00000067,0x00000003,0x00000718,0x00000000,
  0x00000001,0x00000002,0x00000718,0x00000006,0x00000700,0x00000112,0x00000002,0x00000000,
  0x0000070C,0x00000005,0x00000000,0x00000718,0x00000067,0x00000003,0x00000758,0x00000000,
  0xFFFFFFFF,0x00000002,0x00000758,0x00000006,0x00000740,0x00000114,0x00000002,0x00000000,
  0x0000074C,0x00000009,0x00000000,0x00000758,0x00000067,0x00000003,0x00000798,0x00000000,
  0xFFFFFFFF,0x00000002,0x00000798,0x00000006,0x00000780,0x00000116,0x00000002,0x00000000,
  0x0000078C,0x0000000A,0x00000000,0x00000798,0x00000067,0x00000003,0x000007D8,0x00000000,
  0x00000001,0x00000002,0x000007D8,0x00000006,0x000007C0,0x00000118,0x00000002,0x00000000,
  0x000007CC,0x0000000E,0x00000000,0x000007D8,0x00000067,0x00000003,0x00000818,0x00000000,
  0xFFFFFFFF,0x00000002,0x00000818,0x00000006,0x00000800,0x0000011A,0x00000002,0x00000000,
  0x0000080C,0x0000000F,0x00000000,0x00000818,0x00000067,0x00000002,0x000008C0,0x00000000,
  0x0000082C,0x00000012,0x00000000,0x00000838,0x00000064,0x00000000,0x00000844,0x00000015,
  0x00000003,0x00000878,0x00000000,0x00000001,0x00000002,0x00000878,0x00000007,0x0000086C,
  0x00000128,0x00000001,0x00000000,0x00000878,0x00000016,0x00000000,0x00000884,0x00000064,
  0x00000000,0x00000890,0x00000014,0x00000003,0x000008C0,0x00000001,0xFFFFFFFF,0x00000002,
  0x000008C0,0x00000000,0x000008B4,0x00000064,0x00000000,0x000008C0,0x00000013,0x00000000,
  0x000008CC,0x00000041,0x00000000,0x000008D8,0x00000041,0x00000002,0x00000920,0x00000000,
  0x000008EC,0x00000041,0x00000003,0x00000920,0x00000000,0xFFFFFFFF,0x00000002,0x00000920,
  0x00000007,0x00000914,0x00000139,0x00000001,0x00000000,0x00000920,0x00000041,0x00000003,
  0x0000093C,0x00000001,0xFFFFFFFF,0x00000000,0x0000093C,0x00000069,0x00000002,0x0000096C,
  0x00000000,0x00000950,0x0000006D,0x00000003,0x0000096C,0x00000000,0xFFFFFFFF,0x00000000,
  0x0000096C,0x00000069,0x00000000,0x00000978,0x0000003F,0x00000003,0x00000994,0x00000001,
  0xFFFFFFFF,0x00000000,0x00000994,0x00000069,0x00000003,0x000009B0,0x00000001,0xFFFFFFFF,
  0x00000000,0x000009B0,0x00000069,0x00000000,0x000009BC,0x00000041,0x00000000,0x000009C8,
  0x00000041,0x00000001,0x000009F4,0x00000000,0x000009DC,0x0000001D,0x00000000,0x000009E8,
  0x0000001E,0x00000000,0x000009F4,0x0000004B,0x00000002,0x00000A60,0x00000000,0x00000A08,
  0x0000004B,0x00000003,0x00000A24,0x00000001,0xFFFFFFFF,0x00000000,0x00000A24,0x00000064,
  0x00000007,0x00000A34,0x0000019F,0x00000001,0x00000003,0x00000A50,0x00000001,0xFFFFFFFF,
  0x00000000,0x00000A50,0x00000042,0x00000007,0x00000A60,0x000001A0,0x00000001,0x00000002,
  0x00000ACC,0x00000003,0x00000A84,0x00000001,0xFFFFFFFF,0x00000000,0x00000A84,0x00000042,
  0x00000003,0x00000AA0,0x00000001,0xFFFFFFFF,0x00000000,0x00000AA0,0x00000064,0x00000007,
  0x00000AB0,0x000001B6,0x00000001,0x00000000,0x00000ABC,0x0000004B,0x00000007,0x00000ACC,
  0x000001B7,0x00000001,0x00000001,0x00000B9C,0x00000002,0x00000B40,0x00000000,0x00000AE8,
  0x00000020,0x00000003,0x00000B04,0x00000000,0xFFFFFFFF,0x00000000,0x00000B04,0x00000064,
  0x00000007,0x00000B14,0x000001C5,0x00000001,0x00000003,0x00000B30,0x00000001,0xFFFFFFFF,
  0x00000000,0x00000B30,0x00000042,0x00000007,0x00000B40,0x000001C6,0x00000001,0x00000002,
  0x00000B90,0x00000003,0x00000B64,0x00000001,0xFFFFFFFF,0x00000000,0x00000B64,0x00000042,
  0x00000007,0x00000B74,0x000001C7,0x00000001,0x00000000,0x00000B80,0x00000020,0x00000007,
  0x00000B90,0x000001C8,0x00000001,0x00000000,0x00000B9C,0x00000020,0x00000002,0x00000C10,
  0x00000003,0x00000BC4,0x00000000,0x00000001,0x00000007,0x00000BC4,0x000001CF,0x00000001,
  0x00000000,0x00000BD0,0x00000069,0x00000003,0x00000C10,0x00000001,0xFFFFFFFF,0x00000001,
  0x00000C10,0x00000000,0x00000BF4,0x00000064,0x00000007,0x00000C04,0x000001D0,0x00000001,
  0x00000000,0x00000C10,0x00000069,0x00000000,0x00000C1C,0x00000041,0x00000003,0x00000C38,
  0x00000001,0xFFFFFFFF,0x00000000,0x00000C38,0x00000069,0x00000001,0x00000C5C,0x00000000,
  0x00000C4C,0x00000025,0x00000007,0x00000C5C,0x000001ED,0x00000001,0x00000001,0x00000C80,
  0x00000000,0x00000C70,0x00000025,0x00000007,0x00000C80,0x000001F8,0x00000001,0x00000002,
  0x00000CB0,0x00000000,0x00000C94,0x0000006D,0x00000003,0x00000CB0,0x00000009,0xFFFFFFFF,
  0x00000000,0x00000CB0,0x00000069,0x00000002,0x00000CFC,0x00000000,0x00000CC4,0x0000006D,
  0x00000003,0x00000CE0,0x00000000,0xFFFFFFFF,0x00000000,0x00000CE0,0x00000069,0x00000003,
  0x00000CFC,0x00000000,0x00000001,0x00000000,0x00000CFC,0x00000028,0x00000002,0x00000D3C,
  0x00000003,0x00000D20,0x00000001,0xFFFFFFFF,0x00000000,0x00000D20,0x00000069,0x00000003,
  0x00000D3C,0x00000000,0x00000001,0x00000000,0x00000D3C,0x00000028,0x00000001,0x00000D84,
  0x00000005,0x00000D54,0x00000064,0x00000064,0x00000005,0x00000D64,0x00000068,0x00000068,
  0x00000005,0x00000D74,0x0000006D,0x0000006D,0x00000005,0x00000D84,0x00000073,0x00000073,
  0x00000001,0x00000E08,0x00000006,0x00000D9C,0x00000236,0x00000006,0x00000002,0x00000DC0,
  0x00000006,0x00000DB4,0x0000023C,0x00000006,0x00000000,0x00000DC0,0x0000003D,0x00000002,
  0x00000DE4,0x00000006,0x00000DD8,0x00000242,0x00000007,0x00000000,0x00000DE4,0x0000002A,
  0x00000002,0x00000E08,0x00000006,0x00000DFC,0x00000249,0x00000004,0x00000000,0x00000E08,
  0x00000044,0x00000002,0x00000E48,0x00000003,0x00000E2C,0x00000000,0xFFFFFFFF,0x00000000,
  0x00000E2C,0x0000002B,0x00000003,0x00000E48,0x00000000,0x00000001,0x00000000,0x00000E48,
  0x0000002C,0x00000003,0x00000E64,0x00000004,0x00000004,0x00000000,0x00000E64,0x0000002D,
  0x00000001,0x00000ED4,0x00000002,0x00000EA0,0x00000003,0x00000E90,0x00000002,0x00000002,
  0x00000000,0x00000E90,0x0000002D,0x00000007,0x00000EA0,0x0000026B,0x00000002,0x00000002,
  0x00000ED4,0x00000003,0x00000EC4,0x00000003,0x00000003,0x00000000,0x00000EC4,0x0000002D,
  0x00000007,0x00000ED4,0x0000026D,0x00000001,0x00000001,0x00000F14,0x00000000,0x00000EE8,
  0x00000068,0x00000000,0x00000EF4,0x00000069,0x00000007,0x00000F04,0x0000027A,0x00000001,
  0x00000007,0x00000F14,0x0000027B,0x00000001,0x00000001,0x00000F58,0x00000002,0x00000F4C,
  0x00000000,0x00000F30,0x0000002F,0x00000007,0x00000F40,0x00000286,0x00000001,0x00000000,
  0x00000F4C,0x00000030,0x00000000,0x00000F58,0x0000002F,0x00000000,0x00000F64,0x00000041,
  0x00000000,0x00000F70,0x0000003E,0x00000001,0x00000F9C,0x00000000,0x00000F84,0x0000003A,
  0x00000000,0x00000F90,0x00000035,0x00000000,0x00000F9C,0x00000032,0x00000002,0x00000FDC,
  0x00000000,0x00000FB0,0x0000001B,0x00000000,0x00000FBC,0x00000064,0x00000001,0x00000FDC,
  0x00000000,0x00000FD0,0x00000033,0x00000000,0x00000FDC,0x00000034,0x00000003,0x00001020,
  0x00000004,0xFFFFFFFF,0x00000001,0x00001020,0x00000000,0x00001000,0x00000043,0x00000007,
  0x00001010,0x000002BF,0x00000001,0x00000007,0x00001020,0x000002C0,0x00000001,0x00000000,
  0x0000102C,0x0000003F,0x00000002,0x0000106C,0x00000000,0x00001040,0x00000036,0x00000000,
  0x0000104C,0x00000064,0x00000001,0x0000106C,0x00000000,0x00001060,0x00000037,0x00000000,
  0x0000106C,0x00000047,0x00000007,0x0000107C,0x000002E3,0x00000003,0x00000002,0x000010E0,
  0x00000000,0x00001090,0x00000046,0x00000007,0x000010A0,0x000002F3,0x00000001,0x00000000,
  0x000010AC,0x00000038,0x00000003,0x000010E0,0x00000000,0x00000001,0x00000002,0x000010E0,
  0x00000007,0x000010D4,0x000002F4,0x00000001,0x00000000,0x000010E0,0x00000039,0x00000001,
  0x00001128,0x00000002,0x00001118,0x00000000,0x000010FC,0x0000006D,0x00000003,0x00001118,
  0x00000000,0x00000002,0x00000000,0x00001118,0x00000069,0x00000007,0x00001128,0x000002F9,
  0x00000001,0x00000002,0x00001158,0x00000000,0x0000113C,0x0000006D,0x00000003,0x00001158,
  0x00000000,0xFFFFFFFF,0x00000000,0x00001158,0x00000069,0x00000002,0x00001198,0x00000000,
  0x0000116C,0x0000003B,0x00000000,0x00001178,0x00000064,0x00000001,0x00001198,0x00000000,
  0x0000118C,0x0000003C,0x00000000,0x00001198,0x0000004A,0x00000007,0x000011A8,0x0000031F,
  0x00000003,0x00000002,0x000011F0,0x00000000,0x000011BC,0x00000049,0x00000003,0x000011F0,
  0x00000000,0x00000001,0x00000002,0x000011F0,0x00000007,0x000011E4,0x0000032E,0x00000001,
  0x00000000,0x000011F0,0x00000039,0x00000000,0x000011FC,0x0000003E,0x00000003,0x00001244,
  0x00000001,0xFFFFFFFF,0x00000001,0x00001244,0x00000005,0x00001224,0x00000001,0x00000009,
  0x00000005,0x00001234,0x0000000B,0x0000000C,0x00000005,0x00001244,0x0000000E,0x000000FF,
  0x00000003,0x00001278,0x00000001,0xFFFFFFFF,0x00000001,0x00001278,0x00000000,0x00001268,
  0x0000006A,0x00000005,0x00001278,0x00000080,0x000000FF,0x00000001,0x000012F0,0x00000005,
  0x00001290,0x00000021,0x00000021,0x00000005,0x000012A0,0x00000023,0x00000027,0x00000005,
  0x000012B0,0x0000002A,0x0000002B,0x00000005,0x000012C0,0x0000002D,0x0000002E,0x00000005,
  0x000012D0,0x00000030,0x00000039,0x00000005,0x000012E0,0x00000041,0x0000005A,0x00000005,
  0x000012F0,0x0000005E,0x0000007E,0x00000003,0x0000130C,0x00000001,0xFFFFFFFF,0x00000000,
  0x0000130C,0x00000040,0x00000001,0x00001374,0x00000005,0x00001324,0x00000001,0x00000009,
  0x00000005,0x00001334,0x0000000B,0x0000000C,0x00000005,0x00001344,0x0000000E,0x00000027,
  0x00000005,0x00001354,0x0000002A,0x0000003B,0x00000005,0x00001364,0x0000003D,0x0000003D,
  0x00000005,0x00001374,0x0000003F,0x000000FF,0x00000001,0x00001394,0x00000000,0x00001388,
  0x00000068,0x00000000,0x00001394,0x00000069,0x00000003,0x000013B4,0x00000001,0xFFFFFFFF,
  0x00000005,0x000013B4,0x00000021,0x0000007F,0x00000003,0x000013D0,0x00000001,0x00000003,
  0x00000000,0x000013D0,0x00000069,0x00000002,0x00001418,0x00000000,0x000013E4,0x00000048,
  0x00000003,0x00001418,0x00000003,0x00000003,0x00000002,0x00001418,0x00000007,0x0000140C,
  0x0000039E,0x00000001,0x00000000,0x00001418,0x00000045,0x00000002,0x00001460,0x00000000,
  0x0000142C,0x00000045,0x00000003,0x00001460,0x00000003,0x00000003,0x00000002,0x00001460,
  0x00000007,0x00001454,0x000003AB,0x00000001,0x00000000,0x00001460,0x00000045,0x00000001,
  0x000014B4,0x00000002,0x00001490,0x00000007,0x00001480,0x000003B3,0x00000002,0x00000005,
  0x00001490,0x00000034,0x00000039,0x00000002,0x000014B4,0x00000007,0x000014A8,0x000003B5,
  0x00000002,0x00000000,0x000014B4,0x00000069,0x00000002,0x00001510,0x00000007,0x000014CC,
  0x000003C5,0x00000002,0x00000003,0x00001510,0x00000001,0xFFFFFFFF,0x00000001,0x00001510,
  0x00000000,0x000014F0,0x0000006C,0x00000007,0x00001500,0x000003C7,0x00000001,0x00000007,
  0x00001510,0x000003C8,0x00000001,0x00000003,0x00001554,0x00000001,0xFFFFFFFF,0x00000001,
  0x00001554,0x00000000,0x00001534,0x0000006C,0x00000007,0x00001544,0x000003D5,0x00000001,
  0x00000007,0x00001554,0x000003D6,0x00000001,0x00000002,0x00001584,0x00000000,0x00001568,
  0x0000004C,0x00000007,0x00001578,0x000003E1,0x00000001,0x00000000,0x00001584,0x0000004D,
  0x00000001,0x000015B0,0x00000000,0x00001598,0x00000054,0x00000000,0x000015A4,0x00000058,
  0x00000000,0x000015B0,0x0000005B,0x00000001,0x000015DC,0x00000000,0x000015C4,0x00000054,
  0x00000000,0x000015D0,0x0000004E,0x00000000,0x000015DC,0x0000005C,0x00000002,0x00001698,
  0x00000003,0x00001600,0x00000000,0x00000001,0x00000000,0x00001600,0x00000062,0x00000007,
  0x00001610,0x00000403,0x00000001,0x00000003,0x00001650,0x00000000,0xFFFFFFFF,0x00000002,
  0x00001650,0x00000003,0x00001644,0x00000000,0x00000001,0x00000000,0x00001644,0x0000005E,
  0x00000000,0x00001650,0x0000004F,0x00000003,0x0000166C,0x00000000,0x00000001,0x00000000,
  0x0000166C,0x0000005E,0x00000007,0x0000167C,0x00000404,0x00000001,0x00000003,0x00001698,
  0x00000000,0x00000001,0x00000000,0x00001698,0x00000062,0x00000001,0x000016B8,0x00000000,
  0x000016AC,0x00000050,0x00000000,0x000016B8,0x00000059,0x00000001,0x000016EC,0x00000000,
  0x000016CC,0x00000051,0x00000005,0x000016DC,0x00000021,0x0000005A,0x00000005,0x000016EC,
  0x0000005E,0x0000007E,0x00000001,0x00001744,0x00000005,0x00001704,0x00000001,0x00000008,
  0x00000005,0x00001714,0x0000000B,0x0000000B,0x00000005,0x00001724,0x0000000C,0x0000000C,
  0x00000005,0x00001734,0x0000000E,0x0000001F,0x00000005,0x00001744,0x0000007F,0x0000007F,
  0x00000001,0x00001894,0x00000000,0x00001758,0x00000068,0x00000000,0x00001764,0x00000069,
  0x00000007,0x00001774,0x00000424,0x00000001,0x00000007,0x00001784,0x00000425,0x00000001,
  0x00000007,0x00001794,0x00000426,0x00000001,0x00000007,0x000017A4,0x00000427,0x00000001,
  0x00000007,0x000017B4,0x00000428,0x00000001,0x00000007,0x000017C4,0x00000429,0x00000001,
  0x00000007,0x000017D4,0x0000042A,0x00000001,0x00000007,0x000017E4,0x0000042B,0x00000001,
  0x00000007,0x000017F4,0x0000042C,0x00000001,0x00000007,0x00001804,0x0000042D,0x00000001,
  0x00000007,0x00001814,0x0000042E,0x00000001,0x00000007,0x00001824,0x0000042F,0x00000001,
  0x00000007,0x00001834,0x00000430,0x00000001,0x00000007,0x00001844,0x00000431,0x00000001,
  0x00000007,0x00001854,0x00000432,0x00000001,0x00000007,0x00001864,0x00000433,0x00000001,
  0x00000007,0x00001874,0x00000434,0x00000001,0x00000007,0x00001884,0x00000435,0x00000001,
  0x00000007,0x00001894,0x00000436,0x00000001,0x00000002,0x000018F0,0x00000003,0x000018B8,
  0x00000000,0x00000001,0x00000000,0x000018B8,0x00000062,0x00000003,0x000018D4,0x00000001,
  0xFFFFFFFF,0x00000000,0x000018D4,0x00000052,0x00000003,0x000018F0,0x00000000,0x00000001,
  0x00000000,0x000018F0,0x00000062,0x00000002,0x0000193C,0x00000003,0x00001914,0x00000000,
  0x00000001,0x00000000,0x00001914,0x00000062,0x00000000,0x00001920,0x00000055,0x00000003,
  0x0000193C,0x00000000,0x00000001,0x00000000,0x0000193C,0x00000062,0x00000002,0x000019A4,
  0x00000003,0x00001960,0x00000001,0xFFFFFFFF,0x00000000,0x00001960,0x00000052,0x00000003,
  0x000019A4,0x00000000,0xFFFFFFFF,0x00000002,0x000019A4,0x00000007,0x00001988,0x00000453,
  0x00000001,0x00000003,0x000019A4,0x00000001,0xFFFFFFFF,0x00000000,0x000019A4,0x00000052,
  0x00000001,0x000019E8,0x00000000,0x000019B8,0x00000051,0x00000005,0x000019C8,0x00000021,
  0x00000021,0x00000005,0x000019D8,0x00000023,0x0000005B,0x00000005,0x000019E8,0x0000005D,
  0x0000007E,0x00000001,0x00001A08,0x00000000,0x000019FC,0x00000056,0x00000000,0x00001A08,
  0x00000059,0x00000002,0x00001ABC,0x00000003,0x00001A2C,0x00000000,0x00000001,0x00000000,
  0x00001A2C,0x00000062,0x00000000,0x00001A38,0x00000066,0x00000003,0x00001A78,0x00000000,
  0xFFFFFFFF,0x00000002,0x00001A78,0x00000003,0x00001A6C,0x00000000,0x00000001,0x00000000,
  0x00001A6C,0x0000005E,0x00000000,0x00001A78,0x00000057,0x00000003,0x00001A94,0x00000000,
  0x00000001,0x00000000,0x00001A94,0x0000005E,0x00000000,0x00001AA0,0x00000066,0x00000003,
  0x00001ABC,0x00000000,0x00000001,0x00000000,0x00001ABC,0x00000062,0x00000001,0x00001AF4,
  0x00000002,0x00001AE8,0x00000007,0x00001ADC,0x0000047D,0x00000001,0x00000000,0x00001AE8,
  0x0000003D,0x00000000,0x00001AF4,0x0000005A,0x00000002,0x00001B1C,0x00000007,0x00001B0C,
  0x00000485,0x00000001,0x00000005,0x00001B1C,0x00000000,0x0000007F,0x00000002,0x00001B64,
  0x00000000,0x00001B30,0x0000005D,0x00000003,0x00001B64,0x00000000,0xFFFFFFFF,0x00000002,
  0x00001B64,0x00000007,0x00001B58,0x00000495,0x00000001,0x00000000,0x00001B64,0x0000005D,
  0x00000002,0x00001BAC,0x00000000,0x00001B78,0x00000053,0x00000003,0x00001BAC,0x00000000,
  0xFFFFFFFF,0x00000002,0x00001BAC,0x00000007,0x00001BA0,0x000004A1,0x00000001,0x00000000,
  0x00001BAC,0x00000053,0x00000001,0x00001BCC,0x00000000,0x00001BC0,0x00000053,0x00000000,
  0x00001BCC,0x00000058,0x00000001,0x00001C44,0x00000002,0x00001C38,0x00000003,0x00001C1C,
  0x00000000,0x00000001,0x00000002,0x00001C1C,0x00000003,0x00001C10,0x00000000,0xFFFFFFFF,
  0x00000000,0x00001C10,0x0000006B,0x00000000,0x00001C1C,0x00000067,0x00000003,0x00001C38,
  0x00000001,0xFFFFFFFF,0x00000000,0x00001C38,0x0000006B,0x00000000,0x00001C44,0x00000063,
  0x00000001,0x00001C88,0x00000000,0x00001C58,0x00000051,0x00000005,0x00001C68,0x00000021,
  0x00000027,0x00000005,0x00001C78,0x0000002A,0x0000005B,0x00000005,0x00001C88,0x0000005D,
  0x0000007E,0x00000001,0x00001CB4,0x00000000,0x00001C9C,0x0000005F,0x00000000,0x00001CA8,
  0x00000059,0x00000000,0x00001CB4,0x00000061,0x00000002,0x00001D38,0x00000007,0x00001CCC,
  0x000004C2,0x00000001,0x00000003,0x00001D0C,0x00000000,0xFFFFFFFF,0x00000002,0x00001D0C,
  0x00000003,0x00001D00,0x00000000,0x00000001,0x00000000,0x00001D00,0x0000005E,0x00000000,
  0x00001D0C,0x00000060,0x00000003,0x00001D28,0x00000000,0x00000001,0x00000000,0x00001D28,
  0x0000005E,0x00000007,0x00001D38,0x000004C3,0x00000001,0x00000002,0x00001DC4,0x00000003,
  0x00001D80,0x00000000,0xFFFFFFFF,0x00000002,0x00001D80,0x00000003,0x00001D74,0x00000000,
  0x00000001,0x00000000,0x00001D74,0x0000005E,0x00000000,0x00001D80,0x00000061,0x00000001,
  0x00001DC4,0x00000002,0x00001DB8,0x00000003,0x00001DAC,0x00000000,0x00000001,0x00000000,
  0x00001DAC,0x0000005E,0x00000000,0x00001DB8,0x00000061,0x00000000,0x00001DC4,0x0000005E,
  0x00000002,0x00001E28,0x00000003,0x00001DE8,0x00000001,0xFFFFFFFF,0x00000000,0x00001DE8,
  0x0000006B,0x00000003,0x00001E28,0x00000000,0xFFFFFFFF,0x00000002,0x00001E28,0x00000000,
  0x00001E0C,0x00000067,0x00000003,0x00001E28,0x00000001,0xFFFFFFFF,0x00000000,0x00001E28,
  0x0000006B,0x00000005,0x00001E38,0x00000020,0x00000020,0x00000005,0x00001E48,0x00000009,
  0x00000009,0x00000005,0x00001E58,0x00000022,0x00000022,0x00000002,0x00001E80,0x00000005,
  0x00001E70,0x0000000D,0x0000000D,0x00000005,0x00001E80,0x0000000A,0x0000000A,0x00000001,
  0x00001EA8,0x00000005,0x00001E98,0x00000041,0x0000005A,0x00000005,0x00001EA8,0x00000061,
  0x0000007A,0x00000005,0x00001EB8,0x00000030,0x00000039,0x00000005,0x00001EC8,0x00000021,
  0x0000007E,0x00000001,0x00001EE8,0x00000000,0x00001EDC,0x00000064,0x00000000,0x00001EE8,
  0x00000065,0x00000001,0x00001F1C,0x00000000,0x00001EFC,0x00000069,0x00000005,0x00001F0C,
  0x00000041,0x00000046,0x00000005,0x00001F1C,0x00000061,0x00000066,0x00000005,0x00001F2C,
  0x00000031,0x00000039,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x0000000D,
  0x00000000,0x00000030,0x00010000,0x00000001,0x00000035,0x00000000,0x00000308,0x00010000,
  0x00000002,0x00000043,0x00000000,0x00000324,0x00010000,0x00000003,0x00000050,0x00000000,
  0x0000042C,0x00010000,0x00000004,0x00000063,0x00000000,0x00000438,0x00010000,0x00000005,
  0x00000075,0x00000000,0x00000444,0x00010000,0x00000006,0x0000007F,0x00000000,0x00000450,
  0x00010000,0x00000007,0x0000008B,0x00000000,0x0000045C,0x00010000,0x00000008,0x00000097,
  0x00000000,0x00000468,0x00010000,0x00000009,0x000000A8,0x00000000,0x00000494,0x00010000,
  0x0000000A,0x000000B9,0x00000000,0x000004C4,0x00010000,0x0000000B,0x000000C7,0x00000000,
  0x00000578,0x00000000,0xFFFFFFFF,0x000000D5,0x00000000,0x000005D4,0x00000000,0xFFFFFFFF,
  0x000000E8,0x00000000,0x00000688,0x00010000,0x0000000C,0x000000F2,0x00000000,0x00000694,
  0x00010000,0x0000000D,0x00000102,0x00000000,0x000006A0,0x00010000,0x0000000E,0x0000011C,
  0x00000000,0x00000818,0x00010000,0x0000000F,0x00000129,0x00000000,0x000008C0,0x00010000,
  0x00000010,0x0000012F,0x00000000,0x000008CC,0x00010000,0x00000011,0x00000133,0x00000000,
  0x000008D8,0x00010000,0x00000012,0x0000013A,0x00000000,0x00000920,0x00010000,0x00000013,
  0x00000144,0x00000000,0x0000093C,0x00010000,0x00000014,0x0000014E,0x00000000,0x0000096C,
  0x00010000,0x00000015,0x00000157,0x00000000,0x00000978,0x00010000,0x00000016,0x0000015F,
  0x00000000,0x00000994,0x00010000,0x00000017,0x0000016C,0x00000000,0x000009B0,0x00010000,
  0x00000018,0x00000174,0x00000000,0x000009BC,0x00010000,0x00000019,0x0000017D,0x00000000,
  0x000009C8,0x00000000,0xFFFFFFFF,0x0000018B,0x00000000,0x000009F4,0x00000000,0xFFFFFFFF,
  0x000001A1,0x00000000,0x00000A60,0x00000000,0xFFFFFFFF,0x000001B8,0x00000000,0x00000ACC,
  0x00000000,0xFFFFFFFF,0x000001C9,0x00000000,0x00000B9C,0x00000000,0xFFFFFFFF,0x000001D1,
  0x00000000,0x00000C10,0x00000000,0xFFFFFFFF,0x000001D8,0x00000000,0x00000C1C,0x00000000,
  0xFFFFFFFF,0x000001E2,0x00000000,0x00000C38,0x00000000,0xFFFFFFFF,0x000001EE,0x00000000,
  0x00000C5C,0x00000000,0xFFFFFFFF,0x000001F9,0x00000000,0x00000C80,0x00000000,0xFFFFFFFF,
  0x000001FE,0x00000000,0x00000CB0,0x00000000,0xFFFFFFFF,0x0000020E,0x00000000,0x00000CFC,
  0x00000000,0xFFFFFFFF,0x00000219,0x00000000,0x00000D3C,0x00000000,0xFFFFFFFF,0x0000022D,
  0x00000000,0x00000D84,0x00000000,0xFFFFFFFF,0x0000024D,0x00000000,0x00000E08,0x00000000,
  0xFFFFFFFF,0x00000254,0x00000000,0x00000E48,0x00000000,0xFFFFFFFF,0x00000260,0x00000000,
  0x00000E64,0x00000000,0xFFFFFFFF,0x0000026E,0x00000000,0x00000ED4,0x00000000,0xFFFFFFFF,
  0x0000027C,0x00000000,0x00000F14,0x00000000,0xFFFFFFFF,0x00000287,0x00000000,0x00000F58,
  0x00000000,0xFFFFFFFF,0x00000291,0x00000000,0x00000F64,0x00000000,0xFFFFFFFF,0x0000029B,
  0x00000000,0x00000F70,0x00000000,0xFFFFFFFF,0x000002AB,0x00000000,0x00000F9C,0x00000000,
  0xFFFFFFFF,0x000002BA,0x00000000,0x00000FDC,0x00010000,0x0000001A,0x000002C1,0x00000000,
  0x00001020,0x00010000,0x0000001B,0x000002CB,0x00000000,0x0000102C,0x00000000,0xFFFFFFFF,
  0x000002DA,0x00000000,0x0000106C,0x00010000,0x0000001C,0x000002E6,0x00000000,0x0000107C,
  0x00000000,0xFFFFFFFF,0x000002F5,0x00000000,0x000010E0,0x00000000,0xFFFFFFFF,0x000002FA,
  0x00000000,0x00001128,0x00010000,0x0000001D,0x00000307,0x00000000,0x00001158,0x00000000,
  0xFFFFFFFF,0x00000316,0x00000000,0x00001198,0x00010000,0x0000001E,0x00000322,0x00000000,
  0x000011A8,0x00000000,0xFFFFFFFF,0x0000032F,0x00000000,0x000011F0,0x00000000,0xFFFFFFFF,
  0x00000334,0x00000000,0x000011FC,0x00000000,0xFFFFFFFF,0x00000340,0x00000000,0x00001244,
  0x00000000,0xFFFFFFFF,0x0000034E,0x00000000,0x00001278,0x00000000,0xFFFFFFFF,0x00000359,
  0x00000000,0x000012F0,0x00000000,0xFFFFFFFF,0x0000035F,0x00000000,0x0000130C,0x00000000,
  0xFFFFFFFF,0x0000036A,0x00000000,0x00001374,0x00000000,0xFFFFFFFF,0x00000378,0x00000000,
  0x00001394,0x00010000,0x0000001F,0x00000386,0x00000000,0x000013B4,0x00000000,0xFFFFFFFF,
  0x00000390,0x00000000,0x000013D0,0x00010000,0x00000020,0x0000039F,0x00000000,0x00001418,
  0x00010000,0x00000021,0x000003AC,0x00000000,0x00001460,0x00000000,0xFFFFFFFF,0x000003B7,
  0x00000000,0x000014B4,0x00010000,0x00000022,0x000003C9,0x00000000,0x00001510,0x00010000,
  0x00000023,0x000003D7,0x00000000,0x00001554,0x00000000,0xFFFFFFFF,0x000003E2,0x00000000,
  0x00001584,0x00000000,0xFFFFFFFF,0x000003ED,0x00000000,0x000015B0,0x00000000,0xFFFFFFFF,
  0x000003F4,0x00000000,0x000015DC,0x00000000,0xFFFFFFFF,0x00000405,0x00000000,0x00001698,
  0x00000000,0xFFFFFFFF,0x0000040E,0x00000000,0x000016B8,0x00000000,0xFFFFFFFF,0x00000414,
  0x00000000,0x000016EC,0x00000000,0xFFFFFFFF,0x0000041E,0x00000000,0x00001744,0x00000000,
  0xFFFFFFFF,0x00000437,0x00000000,0x00001894,0x00000000,0xFFFFFFFF,0x0000043C,0x00000000,
  0x000018F0,0x00000000,0xFFFFFFFF,0x00000445,0x00000000,0x0000193C,0x00000000,0xFFFFFFFF,
  0x00000454,0x00000000,0x000019A4,0x00000000,0xFFFFFFFF,0x0000045A,0x00000000,0x000019E8,
  0x00000000,0xFFFFFFFF,0x00000463,0x00000000,0x00001A08,0x00000000,0xFFFFFFFF,0x00000471,
  0x00000000,0x00001ABC,0x00000000,0xFFFFFFFF,0x0000047E,0x00000000,0x00001AF4,0x00000000,
  0xFFFFFFFF,0x00000486,0x00000000,0x00001B1C,0x00000000,0xFFFFFFFF,0x00000496,0x00000000,
  0x00001B64,0x00000000,0xFFFFFFFF,0x000004A2,0x00000000,0x00001BAC,0x00000000,0xFFFFFFFF,
  0x000004A7,0x00000000,0x00001BCC,0x00000000,0xFFFFFFFF,0x000004AB,0x00000000,0x00001C44,
  0x00000000,0xFFFFFFFF,0x000004B1,0x00000000,0x00001C88,0x00000000,0xFFFFFFFF,0x000004BA,
  0x00000000,0x00001CB4,0x00000000,0xFFFFFFFF,0x000004C4,0x00000000,0x00001D38,0x00000000,
  0xFFFFFFFF,0x000004C9,0x00000000,0x00001DC4,0x00000000,0xFFFFFFFF,0x000004D1,0x00000000,
  0x00001E28,0x00000000,0xFFFFFFFF,0x000004D4,0x00000000,0x00001E38,0x00000000,0xFFFFFFFF,
  0x000004D9,0x00000000,0x00001E48,0x00000000,0xFFFFFFFF,0x000004E0,0x00000000,0x00001E58,
  0x00000000,0xFFFFFFFF,0x000004E5,0x00000000,0x00001E80,0x00000000,0xFFFFFFFF,0x000004EB,
  0x00000000,0x00001EA8,0x00000000,0xFFFFFFFF,0x000004F1,0x00000000,0x00001EB8,0x00000000,
  0xFFFFFFFF,0x000004F7,0x00000000,0x00001EC8,0x00000000,0xFFFFFFFF,0x000004FB,0x00000000,
  0x00001EE8,0x00000000,0xFFFFFFFF,0x00000502,0x00000000,0x00001F1C,0x00000000,0xFFFFFFFF,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpSDPInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
//}}CallBack.body

// call back function for rule "announcement"
static ulong pfn_announcement(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_announcement
//}}CallBack.pfn_announcement
  return ulReturn;
}

// call back function for rule "session-section"
static ulong pfn_session_section(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_session_section
//}}CallBack.pfn_session_section
  return ulReturn;
}

// call back function for rule "proto-version"
static ulong pfn_proto_version(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_proto_version
//}}CallBack.pfn_proto_version
  return ulReturn;
}

// call back function for rule "origin-field"
static ulong pfn_origin_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_origin_field
//}}CallBack.pfn_origin_field
  return ulReturn;
}

// call back function for rule "session-name-field"
static ulong pfn_session_name_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_session_name_field
//}}CallBack.pfn_session_name_field
  return ulReturn;
}

// call back function for rule "information-field"
static ulong pfn_information_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_information_field
//}}CallBack.pfn_information_field
  return ulReturn;
}

// call back function for rule "uri-field"
static ulong pfn_uri_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_uri_field
//}}CallBack.pfn_uri_field
  return ulReturn;
}

// call back function for rule "email-field"
static ulong pfn_email_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_email_field
//}}CallBack.pfn_email_field
  return ulReturn;
}

// call back function for rule "phone-field"
static ulong pfn_phone_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_phone_field
//}}CallBack.pfn_phone_field
  return ulReturn;
}

// call back function for rule "connection-field"
static ulong pfn_connection_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_connection_field
//}}CallBack.pfn_connection_field
  return ulReturn;
}

// call back function for rule "bandwidth-field"
static ulong pfn_bandwidth_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bandwidth_field
//}}CallBack.pfn_bandwidth_field
  return ulReturn;
}

// call back function for rule "time-fields"
static ulong pfn_time_fields(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_time_fields
//}}CallBack.pfn_time_fields
  return ulReturn;
}

// call back function for rule "key-field"
static ulong pfn_key_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_key_field
//}}CallBack.pfn_key_field
  return ulReturn;
}

// call back function for rule "attribute-field"
static ulong pfn_attribute_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_attribute_field
//}}CallBack.pfn_attribute_field
  return ulReturn;
}

// call back function for rule "media-section"
static ulong pfn_media_section(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_media_section
//}}CallBack.pfn_media_section
  return ulReturn;
}

// call back function for rule "media-field"
static ulong pfn_media_field(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_media_field
//}}CallBack.pfn_media_field
  return ulReturn;
}

// call back function for rule "media"
static ulong pfn_media(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_media
//}}CallBack.pfn_media
  return ulReturn;
}

// call back function for rule "fmt"
static ulong pfn_fmt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_fmt
//}}CallBack.pfn_fmt
  return ulReturn;
}

// call back function for rule "proto"
static ulong pfn_proto(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_proto
//}}CallBack.pfn_proto
  return ulReturn;
}

// call back function for rule "MediaPort"
static ulong pfn_MediaPort(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_MediaPort
//}}CallBack.pfn_MediaPort
  return ulReturn;
}

// call back function for rule "PortCount"
static ulong pfn_PortCount(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_PortCount
//}}CallBack.pfn_PortCount
  return ulReturn;
}

// call back function for rule "username"
static ulong pfn_username(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_username
//}}CallBack.pfn_username
  return ulReturn;
}

// call back function for rule "sess-id"
static ulong pfn_sess_id(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_sess_id
//}}CallBack.pfn_sess_id
  return ulReturn;
}

// call back function for rule "sess-version"
static ulong pfn_sess_version(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_sess_version
//}}CallBack.pfn_sess_version
  return ulReturn;
}

// call back function for rule "nettype"
static ulong pfn_nettype(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_nettype
//}}CallBack.pfn_nettype
  return ulReturn;
}

// call back function for rule "addrtype"
static ulong pfn_addrtype(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_addrtype
//}}CallBack.pfn_addrtype
  return ulReturn;
}

// call back function for rule "FQDN"
static ulong pfn_FQDN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_FQDN
//}}CallBack.pfn_FQDN
  return ulReturn;
}

// call back function for rule "extn-addr"
static ulong pfn_extn_addr(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_extn_addr
//}}CallBack.pfn_extn_addr
  return ulReturn;
}

// call back function for rule "IPv4Type"
static ulong pfn_IPv4Type(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv4Type
//}}CallBack.pfn_IPv4Type
  return ulReturn;
}

// call back function for rule "AddressCount"
static ulong pfn_AddressCount(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_AddressCount
//}}CallBack.pfn_AddressCount
  return ulReturn;
}

// call back function for rule "IPv6Type"
static ulong pfn_IPv6Type(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Type
//}}CallBack.pfn_IPv6Type
  return ulReturn;
}

// call back function for rule "URI-reference"
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_URI_reference
//}}CallBack.pfn_URI_reference
  return ulReturn;
}

// call back function for rule "IPv4Multicast"
static ulong pfn_IPv4Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv4Multicast
  GEN_DATA* spTest = (GEN_DATA*)vpData;
  if(ulState == SEM_PRE && spTest && spTest->ulDataID == RULE_SDP_CONNECTION_FIELD)
  {
    IP_DATA* spData = (IP_DATA*)vpData;
    APG_BSTR sBstr;
    ulong ulIsMulticast;

    // initialize
    ulIsMulticast = spData->ulDataID;
    memset((void*)spData, 0, sizeof(IP_DATA));
    spData->ulDataID = ulIsMulticast;

    // translate with external, hand-coded parser
    sBstr.ucpStr = spData->sSrc.ucpStr + ulOffset;
    sBstr.ulLen = ulLen;
    spData->ulResult = ulIPv4StringToBinary(&sBstr, spData->caIPv4, &ulIsMulticast);
    if(spData->ulResult)
    {
      if(ulIsMulticast)
      {
        spData->ulResult = ulIPv4BinaryToString(spData->caIPv4, spData->caIPv4String);
      }
      else
      {
        spData->ulResult = APG_FALSE;
      }
    }
  }
//}}CallBack.pfn_IPv4Multicast
  return ulReturn;
}

// call back function for rule "IPv4Unicast"
static ulong pfn_IPv4Unicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv4Unicast
  GEN_DATA* spData = (GEN_DATA*)vpData;
  if(ulState == SEM_PRE && spData && spData->ulDataID == RULE_SDP_CONNECTION_FIELD)
  {
  }
//}}CallBack.pfn_IPv4Unicast
  return ulReturn;
}

// call back function for rule "IPv6Multicast"
static ulong pfn_IPv6Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Multicast
  GEN_DATA* spData = (GEN_DATA*)vpData;
  if(ulState == SEM_PRE && spData && spData->ulDataID == RULE_SDP_CONNECTION_FIELD)
  {
  }
//}}CallBack.pfn_IPv6Multicast
  return ulReturn;
}

// call back function for rule "IPv6Unicast"
static ulong pfn_IPv6Unicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Unicast
  GEN_DATA* spData = (GEN_DATA*)vpData;
  if(ulState == SEM_PRE && spData && spData->ulDataID == RULE_SDP_CONNECTION_FIELD)
  {
  }
//}}CallBack.pfn_IPv6Unicast
  return ulReturn;
}

