/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:17
  filename: D:\Projects\Apgv5_0\Samples\SDP\Timing.c
   purpose: Does timing tests.
  
*********************************************************************/
#include "main.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Timing test driver
//
// FUNCTION: void vTestTiming
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 2) char* cpInput
//              : file name of a list of SDP announcements
//
//      ARG: 3) ulong ulRepetitions
//              : the number of times to parse the entire list
//                during the timing.
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void  vTestTiming(void* vpMemCtx, char* cpInput, ulong ulRepetitions)
{
  void*     vpSDPCtx = NULL;
  void*     vpPPTCtx = NULL;
  APG_BSTR  sSrc;
  APG_BSTR* spBstr;
  void*     vpBstr;
  ulong     ulTest;
  ulong     ulCount;
  ulong     ulAnn, ulRep;
  ulong     ulMsgs;
  ulong     ulMsgChars;
  double    dStart;
  double    dFinish;
  double    dClocksPerSec;
  double    dDuration;
  double    dDurationPPT;
  double    dMsgPerMsec;
  double    dCharsPerMsec;
  
  // get an input string
  sSrc.ulLen = ulGetFileSize(cpInput);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  SDPASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  SDPASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  SDPASSERT(vpBstr);
  ulCount = ulReadAnnouncements(&sSrc, vpBstr);
  SDPASSERT(ulCount);
  
  // get a parser components
  vpSDPCtx = vpParserCtor(vpSDPInit, vpMemCtx);
  SDPASSERT(vpSDPCtx);
  vpPPTCtx = vpParserCtor(vpSDPPPTInit, vpMemCtx);
  SDPASSERT(vpPPTCtx);
  
  //////////////////////////////////////////////////////////////////////////
  // SDP test
  //////////////////////////////////////////////////////////////////////////
  // iterate through repetitions
  ulMsgs = 0;
  ulMsgChars = 0;
  dClocksPerSec = (double)CLOCKS_PER_SEC;
  dStart = (double)clock();
  for(ulRep = 0; ulRep < ulRepetitions; ++ulRep)
  {
    // iterate through the announcements
    spBstr = (APG_BSTR*)vpVecFront(vpBstr);
    SDPASSERT(spBstr);
    for(ulAnn = 0; ulAnn < ulCount; ++ulAnn, ++spBstr)
    {
      ++ulMsgs;
      ulMsgChars += spBstr->ulLen;
      ulTest = ulParserSyntaxAnalysis(vpSDPCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
      SDPASSERT(ulTest);
      // assume success, but continue even if not
    }
  }

  // compute rates
  dFinish = (double)clock();
  dDuration = ((dFinish - dStart) * 1000.0) / dClocksPerSec;
  dMsgPerMsec = (double)ulMsgs / dDuration;
  dCharsPerMsec = (double)ulMsgChars / dDuration;
  
  printf("SDP without PPT\n");
  printf(" -      Announcements = %lu\n", ulMsgs);
  printf(" -         Characters = %lu\n", ulMsgChars);
  printf(" -         Time(msec) = %1.4g\n", dDuration);
  printf(" - Announcements/msec = %1.4g\n", dMsgPerMsec);
  printf(" -    Characters/msec = %1.4g\n", dCharsPerMsec);

  //////////////////////////////////////////////////////////////////////////
  // PPT test
  //////////////////////////////////////////////////////////////////////////
  // iterate through repetitions
  ulMsgs = 0;
  ulMsgChars = 0;
  dClocksPerSec = (double)CLOCKS_PER_SEC;
  dStart = (double)clock();
  for(ulRep = 0; ulRep < ulRepetitions; ++ulRep)
  {
    // iterate through the announcements
    spBstr = (APG_BSTR*)vpVecFront(vpBstr);
    SDPASSERT(spBstr);
    for(ulAnn = 0; ulAnn < ulCount; ++ulAnn, ++spBstr)
    {
      ++ulMsgs;
      ulMsgChars += spBstr->ulLen;
      ulTest = ulParserSyntaxAnalysis(vpPPTCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
      SDPASSERT(ulTest);
      // assume success, but continue even if not
    }
  }
  
  // compute rates
  dFinish = (double)clock();
  dDurationPPT = ((dFinish - dStart) * 1000.0) / dClocksPerSec;
  dMsgPerMsec = (double)ulMsgs / dDurationPPT;
  dCharsPerMsec = (double)ulMsgChars / dDurationPPT;
 
  printf("\n");
  printf("SDP with PPT\n");
  printf(" -      Announcements = %lu\n", ulMsgs);
  printf(" -         Characters = %lu\n", ulMsgChars);
  printf(" -         Time(msec) = %1.4g\n", dDurationPPT);
  printf(" - Announcements/msec = %1.4g\n", dMsgPerMsec);
  printf(" -    Characters/msec = %1.4g\n", dCharsPerMsec);

  // compute PPT factor
  printf("\n");
  printf(" -         PPT factor = %1.4g\n", dDuration/dDurationPPT);

  // clean up
  if(vpSDPCtx){vParserDtor(vpSDPCtx);}
  if(vpPPTCtx){vParserDtor(vpPPTCtx);}
}

