/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 8:58
  filename: D:\Projects\Apgv5_0\Samples\URI\Accuracy.c
  purpose: Translates a list of IPv6 addresses. Discovers
  malformed addresses and prints correct ones
                in a normal format.
  
*********************************************************************/
#include "main.h"

static void vPrintOperatorLine(char* cpOp, APG_NODE_STATS* spNode, APG_NODE_STATS* spTotal);
static char cConvert(ushort  usValue);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Test the accuracy of the syntax analysis.
//
// FUNCTION: void vTestAccuracy
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 2) char* cpFileName
//              : name of a file with a list of URIs
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vTestAccuracy(void* vpMemCtx, char* cpFileName)
{
  APG_BSTR          sSrc;
  void*             vpBstr;
  ulong             ulCount;
  TESTDATA          sData;

  // some valid and malformed IPv6 address
  sSrc.ulLen = ulGetFileSize(cpFileName);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  MAINASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpFileName, (void*)sSrc.ucpStr);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  MAINASSERT(vpBstr);
  ulCount = ulReadUris(&sSrc, vpBstr);
  MAINASSERT(ulCount);
  
  sData.vpMemCtx = vpMemCtx;
  sData.vpMsgs = vpBstr;
  sData.ulRepetitions = 1;

  // syntax test with PPT
  sData.vpInit = vpURIInit;
  sData.cpTitle = "//////////////////////////////////////////////////////////////////////////\n"\
    "// SYNTAX ACCURACY TEST\n"\
    "//////////////////////////////////////////////////////////////////////////";
  vRunAccuracy(&sData);

  // clean up resources
  vVecDtor(vpBstr);
  vMemFree(vpMemCtx, sSrc.ucpStr);
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Perform the actual tests.
//
// FUNCTION: void vRunAccuracy
//
//      ARG: 1) TESTDATA* spTestData
//              : pointer to a specialized data struct
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vRunAccuracy(TESTDATA* spTestData)
{
  APG_PARSER_STATE  sState;
  APG_BSTR*         spBstr;
  ulong             ulTest;
  ulong             ulCount;
  ulong             ulAdd;
  URI_DATA          sData;
  void*             vpParserCtx;
  char              caBuffer[1024];
  DASSERT(spTestData);
  DASSERT(spTestData->vpMemCtx);
  DASSERT(spTestData->vpMsgs);
  DASSERT(spTestData->vpInit);
  
  memset((void*)&spTestData->sStats, 0, sizeof(spTestData->sStats));
  if(spTestData->cpTitle && *spTestData->cpTitle != 0)
  {
    // print a header
    printf("\n");
    printf("%s\n", spTestData->cpTitle);
  }
  
  // get the list of addresses
  spBstr = (APG_BSTR*)vpVecFront(spTestData->vpMsgs);
  MAINASSERT(spBstr);
  ulCount = ulVecSize(spTestData->vpMsgs);
  MAINASSERT(ulCount);
  
  // initialize the parser context
  ulTest = ulParserInit(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(ulTest);
  vpParserCtx = vpParserCtor(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(vpParserCtx);
  
  // translate each address
  for(ulAdd = 0; ulAdd < ulCount; ++ulAdd, ++spBstr)
  {
    memset((void*)&sData, 0, sizeof(URI_DATA));
    ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
    MAINASSERT(ulTest);
    ulTest = ulParserState(vpParserCtx, &sState);
    if(ulTest && sState.ulState == MATCH)
    {
      // translate
      memset((void*)&sData, 0, sizeof(URI_DATA));
      sData.sSrc = *spBstr;
      ulTest = ulParserSemanticAnalysis(vpParserCtx, (void*)&sData);
      if(ulTest && sData.ulErrorCode == URI_OK)
      {
        // original URI-reference
        ulCatalogGetString(&sData.sSrc, caBuffer);
        printf("\n");
        printf("[%03lu]      original - %s\n", ulAdd, (char*)sData.sSrc.ucpStr);

        switch(sData.ulUri)
        {
        case URI_URI:
          // scheme
          ulCatalogGetString(&sData.sScheme, caBuffer);
          printf("             scheme - %s\n", caBuffer);
        	break;
        case URI_RELATIVE:
          printf("             scheme - <none - relative reference>\n");
          break;
        default:
          MAINASSERT(APG_FALSE);
          break;
        }

        // authority
        if(sData.sAuthority.ulLen == 0)
        {
          printf("          authority - <empty>\n");
        }
        else
        {
          ulCatalogGetString(&sData.sAuthority, caBuffer);
          printf("          authority - %s\n", caBuffer);
        }

        // path
        switch(sData.ulPath)
        {
        case PATH_EMPTY:
          printf("               path - <empty>\n");
          break;
        case PATH_NOSCHEME:
          ulCatalogGetString(&sData.sPath, caBuffer);
          printf("               path - %s (noscheme)\n", caBuffer);
        	break;
        case PATH_ABSOLUTE:
          ulCatalogGetString(&sData.sPath, caBuffer);
          printf("               path - %s (absolute)\n", caBuffer);
        	break;
        case PATH_ROOTLESS:
          ulCatalogGetString(&sData.sPath, caBuffer);
          printf("               path - %s (rootless)\n", caBuffer);
            break;
        case PATH_ABEMPTY:
          if(sData.sPath.ulLen == 0)
          {
            printf("               path - <empty> (abempty)\n");
          }
          else
          {
            ulCatalogGetString(&sData.sPath, caBuffer);
            printf("               path - %s (abempty)\n", caBuffer);
          }
            break;
        default:
            break;
        }

        // query
        if(sData.sQuery.ulLen > 0)
        {
          ulCatalogGetString(&sData.sQuery, caBuffer);
          printf("              query - %s\n", caBuffer);
        }
        else
        {
          printf("              query - <empty>\n");
        }
        
        // fragment
        if(sData.sFragment.ulLen > 0)
        {
          ulCatalogGetString(&sData.sFragment, caBuffer);
          printf("           fragment - %s\n", caBuffer);
        }
        else
        {
          printf("           fragment - <empty>\n");
        }
        
      }
    }
    else
    {
      sData.ulErrorCode = URI_SYNTAX;
    }

    if(sData.ulErrorCode != URI_OK)
    {
      // print error
      printf("\n");
      printf("[%03lu]     malformed - %s\n", ulAdd, (char*)spBstr->ucpStr);
      printf("         error code - %lu\n", sData.ulErrorCode);
    }
  }

  // clean up
  vParserDtor(vpParserCtx);
}
