/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 10:25
  filename: D:\Projects\Apgv5_0\ApgLib\Apg.h
   purpose: The one and only public include file required for
            APG static library and generated parser use.
  
*********************************************************************/
#ifndef _Apg_h_15_6_2006_9_36_
#define _Apg_h_15_6_2006_9_36_

#include <stdlib.h>
#include <string.h>

// Begin C-language linkage specification
#if defined(__cplusplus)
extern "C"
{
#endif

//////////////////////////////////////////////////////////////////////////
// CONFIGURATION CONTROL
// Control of debugging and statistics information is designed
// through macros so that when not needed, they generate no code.
//
// Debug information
// _APG_CFG_DEBUG - if defined
//                - defines debug assertion (DASSERT) and parser tracing.
//                - automatically defines _APG_CFG_STATS
//
// statistics gathering
// _APG_CFG_STATS - if defined, turns on statistics collection (except Packrat)
// _APG_CFG_PACKRAT - if defined, turns on parser Packrat statistics collection
//////////////////////////////////////////////////////////////////////////

// Configurable default values
#undef APG_ERR_DEFAULT
#undef APG_ERROR_BUFLEN
#undef APG_VEC_ELEMENT
#undef APG_VEC_ALLOC
#define APG_ERR_DEFAULT   100   // default low-level error message buffer size
#define APG_ERROR_BUFLEN  256   // high-level error reporting, max text message length
#define APG_VEC_ELEMENT   1     // default vector element size (bytes)
#define APG_VEC_ALLOC     100   // default initial number of vector elements allocated

//////////////////////////////////////////////////////////////////////////
// APG ERROR IDENTIFIERS
// APG-defined error IDs are all negative
// User-defined error IDs should all be positive to avoid conflict
//////////////////////////////////////////////////////////////////////////
#undef APG_DBG_ID
#undef APG_MEM_ID
#undef APG_VEC_ID
#undef APG_PAR_ID
#undef APG_CAT_ID
#undef APG_GEN_ID
#define APG_DBG_ID  (-1)  // debug assertion error
#define APG_MEM_ID  (-2)  // memory component procedure error
#define APG_VEC_ID  (-3)  // vector component procedure error
#define APG_PAR_ID  (-4)  // Parser component procedure error
#define APG_CAT_ID  (-5)  // Catalog component procedure error
#define APG_GEN_ID  (-6)  // generator (Apg.exe) procedure error

//////////////////////////////////////////////////////////////////////////
// DEBUG, ASSERT, TRACING
//////////////////////////////////////////////////////////////////////////
// always defined
// user assert - allows user to define the identifier and  error(memory) component location
//               calling function must have an APG_EXIT label defined
#undef  UASSERT
#define UASSERT(cond, err, id) if(!(int)(cond)) {ulErrReport((err), (id), __LINE__, __FILE__); goto APG_EXIT;}

// calls the alert handler with an alert identifier
#undef  ALERTASSERT
#define ALERTASSERT(cond, id) if(!(int)(cond)) {vAlertHandler((id), __LINE__, __FILE__);}

// these macros generate no code unless otherwise defined
#define DASSERT(cond)
#define APG_MEMSTATS(ctx, ID, incell, outcell)
#define APG_NODESTATSBEG(stats)
#define APG_NODESTATSEND(stats, op, c,state, m, t)
#define APG_NODESTATSEOF(x, c, state)
#define APG_NODETRACEBEG(d,c,o)
#define APG_NODETRACEEND(d,c,o,s,m)
#define APG_NODESTATS(ctx, func, type, ch, state, match)
#define APG_PACKRATCTOR(ctx)
#define APG_PACKRATDTOR(ctx)
#define APG_PACKRATSTATS(ctx, rule, char, state, match)

#if defined(_APG_CFG_DEBUG)
  // debug also defines statistics gathering
  #undef  _APG_CFG_STATS
  #define _APG_CFG_STATS

  // debug assertion
  #undef  DASSERT
  #define DASSERT(cond) if(!(int)(cond)) vAlertHandler(APG_DBG_ID, __LINE__, __FILE__)

  // tracing macros (requires run-time switch in addition to _APG_CFG_DEBUG)
  #undef  APG_NODETRACEBEG
  #undef  APG_NODETRACEEND
  #define APG_NODETRACEBEG(d,c,o) vNodeTraceBegin((d), (c), (o)) 
  #define APG_NODETRACEEND(d,c,o,s,m) vNodeTraceEnd((d), (c), (o), (s), (m)) 
#endif

#if defined(_APG_CFG_STATS)
  // statistics gathering
  #undef  APG_MEMSTATS
  #undef  APG_NODESTATSBEG
  #undef  APG_NODESTATSEND
  #undef  APG_NODESTATSEOF
  #undef  APG_NODESTATS
  #define APG_MEMSTATS(ctx, ID, incell, outcell) vMemStats((ctx), (ID), (incell), (outcell))
  #define APG_NODESTATSBEG(stats) vNodeStatsBegin(stats)
  #define APG_NODESTATSEND(stats, op, c,state, m, t) vNodeStatsEnd((stats), (op), (c), (state), (m), (t))
  #define APG_NODESTATSEOF(x, c, state) vNodeStatsEOF((x), (c), (state));
  #define APG_NODESTATS(ctx, func, type, ch, state, match) vNodeStats((ctx), (func), (type), (ch), (state), (match))

  // node statistics function identifiers
  #undef  OP_ID_PPT
  #undef  OP_ID_EOF
  #undef  NODE_BEGIN
  #undef  NODE_END
  #undef  NODE_CHILD_COUNT
  #undef  NODE_EOF_VISIT
  #undef  NODE_EOF_ACCEPT
  #undef  NODE_PPT
  #undef  NODE_BACKTRACK
  #define OP_ID_PPT         (OP_ID_MAX+1)
  #define OP_ID_EOF         (OP_ID_MAX+2)
  #define NODE_BEGIN        (OP_ID_MAX+3)
  #define NODE_END          (OP_ID_MAX+4)
  #define NODE_CHILD_COUNT  (OP_ID_MAX+5)
  #define NODE_EOF_VISIT    (OP_ID_MAX+6)
  #define NODE_EOF_ACCEPT   (OP_ID_MAX+7)
  #define NODE_PPT          (OP_ID_MAX+8)
  #define NODE_BACKTRACK    (OP_ID_MAX+9)
#endif

#if defined(_APG_CFG_PACKRAT)
  #undef  APG_PACKRATCTOR
  #undef  APG_PACKRATDTOR
  #undef  APG_PACKRATSTATS
  #define APG_PACKRATCTOR(ctx) vpPackratCtor((ctx))
  #define APG_PACKRATDTOR(ctx) vPackratDtor((ctx))
  #define APG_PACKRATSTATS(ctx, rule, char, state, match) vPackratCollect((ctx), (rule), (char), (state), (match))
#endif

//////////////////////////////////////////////////////////////////////////
// BASIC TYPES
//////////////////////////////////////////////////////////////////////////
typedef unsigned char   uchar;
typedef unsigned short  ushort;
typedef unsigned long   ulong;

// non-ANSI minimum/maximum macros
#undef min
#undef max
#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))

// ulong true/false values
#undef  APG_TRUE
#undef  APG_FALSE
#define APG_TRUE   1UL
#define APG_FALSE  0UL

// often used to represent an undefined or infinite integer, etc.
#undef  UNDEFINED
#define UNDEFINED 0xFFFFFFFF

// binary string pointer (fixed - pointer to a fixed address)
typedef struct 
{
  uchar*  ucpStr;
  ulong   ulLen;
} APG_BSTR;

// binary string location (relocatable - offset from an unspecified address)
typedef struct 
{
  ulong ulOffset;
  ulong ulLen;
} APG_BLOC;

//////////////////////////////////////////////////////////////////////////
// TOOLS
//////////////////////////////////////////////////////////////////////////

// CYCLIC REDUNDANCY CHECK
ulong ulCRC32(uchar *buffer, ulong length);
ulong ulcrc32(uchar *buffer, ulong length);
ulong ulChainedCRC32(ulong ulPrevValue, uchar *buffer, ulong length);
ulong ulChainedCRC32Init();

//////////////////////////////////////////////////////////////////////////
// ALERTS
// Reports location of terminal errors.
// - can be customized with pfnSetAlertHandler, recommended
// - default alert handler is exit(0)
//////////////////////////////////////////////////////////////////////////
//
// NOTE: for C structured exception handling (access violations, division by zero, etc.)
//       add (OS-dependent) exception handlers that call vAlertHandler()
//       Highly recommended!
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// ALERT HANDLER
//////////////////////////////////////////////////////////////////////////
typedef void (*PFN_ALERT)(int iID, unsigned int uiLine, char* cpFile);
void vAlertHandler(int iID, unsigned int uiLine, char* cpFile);
PFN_ALERT pfnSetAlertHandler(PFN_ALERT pfnNewAlertHandler);

//////////////////////////////////////////////////////////////////////////
// MEMORY MANAGEMENT COMPONENT
// provides garbage collection
//////////////////////////////////////////////////////////////////////////
typedef void* (*PFN_ALLOCATOR)(size_t);
typedef void  (*PFN_DEALLOCATOR)(void*);
void*   vpMemCtorA(PFN_ALLOCATOR pfnAllocator, PFN_DEALLOCATOR pfnDeAllocator);
void*   vpMemCtor();
void    vMemDtor(void* vpCtx);
void*   vpMemAlloc(void* vpCtx, ulong ulBytes);
void*   vpMemRealloc(void* vpCtx, void* vpData, ulong ulBytes);
void    vMemFree(void* vpCtx, void* vpData);
ulong   ulMemCheckPoint(void* vpCtx);
void    vMemFreeToCheckPoint(void* vpCtx, ulong ulChk);
void    vMemClear(void* vpCtx); // same as vMemFreeToCheckPoint(vpCtx, 0);
//ulong   ulMemPrintStats(void* vpCtx);

//////////////////////////////////////////////////////////////////////////
// ERROR HISTORY COMPONENT
// records source file and line number of APG system errors
//////////////////////////////////////////////////////////////////////////
// basic unit of error-reporting information
typedef struct
{
  ulong ulID;   // error ID, APG_MEM_ID, APG_PROC_ID or user defined
  ulong ulLine; // line number where error occurred
  char* cpFile; // source code file name where error occurred
} APG_ERR;

void  vErrClear(void* vpCtx);
ulong ulErrReport(void* vpCtx, ulong ulID, ulong ulLine, char* cpLine);
ulong ulErrMsg(void* vpCtx, ulong ulIndex, APG_ERR* spError);
ulong ulErrCount(void* vpCtx);
ulong ulErrTotal(void* vpCtx);

//////////////////////////////////////////////////////////////////////////
// VECTOR
// a dynamic buffer for an arbitrary number of fixed-size elements
//////////////////////////////////////////////////////////////////////////
void* vpVecCtor(void* vpMemCtx, ulong ulElementSize, ulong ulInitialAlloc);
void  vVecDtor(void* vpCtx);
void* vpVecPush(void* vpVec, void* vpElement);
void* vpVecPushn(void* vpCtx, void* vpElement, ulong ulCount);
void* vpVecPop(void* vpCtx);
void* vpVecPopn(void* vpCtx, ulong ulCount);
void* vpVecPopTo(void* vpCtx, ulong ulIndex);
void* vpVecFront(void* vpCtx);
void* vpVecBack(void* vpCtx);
void* vpVecAt(void* vpCtx, ulong ulIndex);
ulong ulVecBytes(void* vpCtx);
ulong ulVecSize(void* vpCtx);
ulong ulVecElementSize(void* vpCtx);
ulong ulVecReallocations(void* vpCtx);
void  vVecClear(void* vpCtx);
ulong ulVecExport(void* vpCtx, void* vpBuffer);
void* vpVecImport(void* vpMemCtx, void* vpData);
//ulong ulVecPopFromTo(void* vpCtx, ulong ulFromIndex, ulong ulToIndex);

//////////////////////////////////////////////////////////////////////////
// ERROR REPORTING FACILITY
//////////////////////////////////////////////////////////////////////////
typedef struct
{
  ulong ulLineIndex;
  ulong ulMsgIndex;
} APG_ERROR_ITEM;

void* vpErrorsCtor(void* vpMemCtx);
void  vErrorsDtor(void* vpErrorsCtx);
ulong ulErrorsReport(void* vpErrorsCtx, char* cpMsg);
ulong ulErrorsCount(void* vpErrorsCtx);
void  vErrorsClear(void* vpErrorsCtx);
void  vErrorsInit(void* vpErrorsCtx);
ulong ulErrorsNext(void* vpErrorsCtx, char* cpErrMsg);

//////////////////////////////////////////////////////////////////////////
// PARSER
//////////////////////////////////////////////////////////////////////////

// call back function prototype
// void* vpData     - pointer to optional user-defined data
// ulong ulState    - the syntax or semantic analysis state
// ulong ulOffset   - offset to the first character of the input string being matched
// ulong ulLen      - length of the matched sub-string
// return value     - default is PFN_OK
//                    user may also return PFN_SKIP or PFN_ERROR
typedef ulong (*PFN_CALLBACK)(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// call back function states
// syntax analysis
#define SYN_NOMATCH 0 // post-branch, string was not matched by grammar syntax (NOTE: memset(0) defaults to NOMATCH)
#define SYN_EMPTY   1 // post-branch, syntax was matched by an empty string
#define SYN_MATCH   2 // post-branch, syntax was matched with a string of one or more characters
#define SYN_PRE     3 // pre-branch, set up before matching process begins

// semantic analysis
#define SEM_PRE     4 // pre-branch, set up before branch below current node is traversed
#define SEM_POST    5 // post-branch, return after branch below this node has been traversed

// call back function return values
#define PFN_OK      0 // normal return
#define PFN_SKIP    1 // return from SYN_PRE - treat as SYN_NOMATCH and skip processing of branch below
                      // return from SEM_PRE - skip analysis of the branch below
#define PFN_ERROR   2 // value to use when user needs to reports an error

// operator states
#define NOMATCH SYN_NOMATCH // pattern match failed - 0 characters matched
#define EMPTY   SYN_EMPTY   // pattern match succeeded - 0 characters matched
#define MATCH   SYN_MATCH   // pattern match succeeded - 1 or more characters accepted
#define ACTIVE  SYN_PRE     // pre-branch, before pattern matching begins

// parser final state
typedef struct 
{
  ulong ulState;
  ulong ulCharactersMatched;
  ulong ulStringLength;
} APG_PARSER_STATE;

// used to pass initialization data from parser to application
typedef struct 
{
  ulong   ulInitialized;    // true after vParseInit() has been called once for this data
  void*   vpCallbacks;      // pointer to the call back function list
  void*   vpExeData;        // pointer to the executable data
} APG_PARSER_INIT;

// parsing statistics
typedef struct 
{
//  ulong ulIndex;            // used for sorting
  ulong ulVisits;           // number of node-type visits
  ulong ulNotMatched;       // number of times node-type not matched
  ulong ulEmpty;            // number of times node-type matched empty
  ulong ulMatched;          // number of times node-type matched
  ulong ulPossibleMatched;  // number of times PPT returned "possible"
  ulong ulCharsMatched;     // number of characters matched at this node-type
  ulong ulChildCount;       // number of ALT, CAT or REP children tested
} APG_NODE_STATS;

// NOTE: ulCharsMaxMatch is approximate:
//   - it fails to be accurate in some cases
//   - this will give an idea of where the syntax failed on a NOMATCH
typedef struct
{
  // DYNAMIC HELPERS
  ulong ulNodesDepth;       // stack or grammar tree depth
  ulong ulIndent;           // indentation to use when tracing
  
  // FINAL STATISTICS
  ulong ulNodesMaxDepth;    // maximum stack depth reached at any time
  ulong ulCharsMaxMatch;    // maximum number of characters matched at any time
  
  // BACKTRACKING
  ulong ulBackTrackCount;   // counts the number of times back tracking occurred
  ulong ulBackTrackChars;   // counts the total number of characters back tracked over
  
  // node-type stats
  APG_NODE_STATS sRnm;
  APG_NODE_STATS sAlt;
  APG_NODE_STATS sCat;
  APG_NODE_STATS sRep;
  APG_NODE_STATS sPrd;
  APG_NODE_STATS sTrg;
  APG_NODE_STATS sTls;
  APG_NODE_STATS sTbs;
  APG_NODE_STATS sPpt;
  APG_NODE_STATS sEof;
  APG_NODE_STATS sTotal;
} APG_PARSER_STATS;

// parser member functions
ulong ulParserInit(void* vpParserInit, void* vpMemCtx);
void* vpParserCtor(void* vpParserInit, void* vpMemCtx);
void  vParserDtor(void* vpCtx);
ulong ulParserSyntaxAnalysis(void* vpCtx, ulong ulStartRule, uchar* ucpSrc, ulong ulSrcLen, void* vpData, ulong ulTrace);
ulong ulParserSemanticAnalysis(void* vpCtx, void* vpData);
ulong ulParserState(void* vpCtx, APG_PARSER_STATE* spState);
ulong ulParserStats(void* vpCtx, APG_PARSER_STATS* spStats);

//////////////////////////////////////////////////////////////////////////
// CATALOGING
// alternative view of the AST
//////////////////////////////////////////////////////////////////////////
typedef struct
{
  // for catalog component use only
  void**    vppList;      // the list this iterator iterates on
  ulong     ulCount;      // the number of phrases in the list
  ulong     ulIndex;      // index of the current member of the list

  // defines the phrase for the current member of the list
  ulong     ulParentID;   // rule name ID of the parent rule
  char*     cpParentName; // ASCII string rule name of the parent rule
  ulong     ulPhraseID;   // rule name ID of the phrase
  char*     cpPhraseName; // ASCII string rule name of the phrase
  APG_BSTR  sPhrase;      // the actual phrase as a binary string 
  ulong     ulPhraseIdentifier; // unique-to-catalog identifier, unique for each individual phrase
  ulong     ulParentIdentifier; // unique identifier of the parent phrase
} APG_CAT_ITERATOR;

void*   vpCatalogCtor(void* vpParserCtx);
void    vCatalogDtor(void* vpCatalogCtx);
void    vCatalogClear(void* vpCatalogCtx);
ulong   ulCatalog(void* vpCatalogCtx);
ulong   ulCatalogTreeTraversal(void* vpCatalogCtx, ulong ulNodeIdentifier, void* vpData);
APG_CAT_ITERATOR* spCatalogGetPhraseList(void* vpCatalogCtx, ulong ulID, APG_CAT_ITERATOR* spIter);
APG_CAT_ITERATOR* spCatalogGetChildList(void* vpCatalogCtx, APG_CAT_ITERATOR* spPhraseIter, APG_CAT_ITERATOR* spIter);
APG_CAT_ITERATOR* spCatalogIterBegin(void* vpCatalogCtx, APG_CAT_ITERATOR* spIter);
APG_CAT_ITERATOR* spCatalogIterNext(void* vpCatalogCtx, APG_CAT_ITERATOR* spIter);
ulong   ulCatalogGetString(APG_BSTR* spBstr, char* cpBuf);

// End C-language linkage specification
#if defined(__cplusplus)
}
#endif

#endif // _Apg_h_15_6_2006_9_36_
