/*! \file LSStep.cpp
 <pre>
 <b>File:</b> LSStep.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> Step skill
 </pre>
 */

#include "LSStep.h"

LSStep::LSStep(WorldModel * _world, PlayerModel * _player):LowSkill(_world, _player) {
}

int LSStep::skillType() {
    
    return SKILL_LS_STEP;
}

int LSStep::calculate(Skill ** lowerSkill) {
    
	double timeout = 20.0;
	double speed_ratio_startposition = 3.0;
	double shoulder_angle_delta = 40.0;
	
	SKILL_BEGIN(0)
    
	SKILL_BEGIN_PHASE(1)
        SKILL_CALCULATE(hipR->moveTo(0,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(0,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(0,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(0,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(2)
        SKILL_CALCULATE(ankleR->moveTo(0,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));		
        SKILL_CALCULATE(hipL->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderR->moveTo(shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderL->moveTo(-1*shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        finishedEtape = true;
        SKILL_CALCULATE(hipR->moveTo(-10,speed_ratio_startposition,-15.0,speed_ratio_startposition*3,false, APPROXIMATING, timeout));
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(3)
        SKILL_CALCULATE(hipR->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_WAIT(4.0)
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(4)
        SKILL_CALCULATE(hipR->moveTo(-10,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(10,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(10,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(-10,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(5)
        SKILL_CALCULATE(ankleL->moveTo(0,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipR->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderR->moveTo(-1*shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderL->moveTo(shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        finishedEtape = true;
        SKILL_CALCULATE(hipL->moveTo(-10,speed_ratio_startposition,15.0,speed_ratio_startposition*3,false, APPROXIMATING, timeout));
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(6)
        SKILL_CALCULATE(hipR->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(kneeL->moveTo(0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(kneeR->moveTo(0,speed_ratio_startposition,false, APPROXIMATING, timeout));		
        SKILL_WAIT(4.0)
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(7)
        SKILL_CALCULATE(hipR->moveTo(10,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(-10,speed_ratio_startposition,10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleR->moveTo(-10,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(10,speed_ratio_startposition,-10.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
	SKILL_END_PHASE
    
	SKILL_BEGIN_PHASE(8)
        SKILL_CALCULATE(ankleR->moveTo(0,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(hipL->moveTo(10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(ankleL->moveTo(-10,speed_ratio_startposition,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderR->moveTo(shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        SKILL_CALCULATE(shoulderL->moveTo(-1*shoulder_angle_delta,speed_ratio_startposition*3,0.0,speed_ratio_startposition,false, APPROXIMATING, timeout));
        finishedEtape = true;
        SKILL_CALCULATE(hipR->moveTo(-10,speed_ratio_startposition,-15.0,speed_ratio_startposition*3,false, APPROXIMATING, timeout));
        if (finishedEtape) phase = 2;
	SKILL_END_PHASE
	
	SKILL_END(9)
}
