/*! \file Logger.h
 <pre>
 <b>File:</b> Logger.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> This file contains logger class.
 </pre>
 */

#include "Logger.h"
#include <stdio.h>
#include <fstream>

bool Logger::switches[LOG_COUNT];

//constructor
Logger::Logger(char* filename) {
	
	for (int i=0; i<= LOG_COUNT; i++) {
		switches[i] = false;
	}
    
	f_descr = fopen(filename,"w");
}

//destructor
Logger::~Logger() {
    
	fclose(f_descr);
	delete(pInstance);
}

Logger* Logger::pInstance = 0;
FILE* Logger::f_descr = 0;

void Logger::LoggerInit(char* filename) {
	
	if (!pInstance) {
		pInstance = new Logger(filename);
		pInstance->SetSwitches();
	}	
}

//sets all switches to ON
void Logger::SetSwitches() {
    
	switches[LOG_RECIEVED_DATA] = false;
	switches[LOG_SENT_DATA] = false;
	switches[LOG_PARSE_TEST] = false;
	switches[LOG_COMMUNICATION] = false;
	switches[LOG_CONFIG] = false;
	switches[LOG_THREADS] = false;
	switches[LOG_TEMP] = true;
	
	switches[LOG_WORLD_BALL_POSITION] = false;
    
	switches[LOG_PLAYER_SIMTIME] = false;
	switches[LOG_PLAYER_GYROSCOPE] = false;
	switches[LOG_PLAYER_FRP] = false;
	
	switches[LOG_PLAYER_ANKLES] = false;
	switches[LOG_PLAYER_KNEES] = false;
	switches[LOG_PLAYER_LEGS] = false;
	switches[LOG_PLAYER_HIPS] = false;
	switches[LOG_PLAYER_SHOULDERS] = false;
	switches[LOG_PLAYER_ELBOWS] = false;
	switches[LOG_PLAYER_ARMS] = false;
}

//sets all switches to ON
void Logger::SetSwitch(int swich_id, bool value) {
    
	switches[swich_id] = value;
}

void Logger::logoutput(int switch_id, const char *txt, ...) {
	
	if (pInstance && pInstance->switches[switch_id] == true) {
		
		static char buf[1024];
        
		va_list argptr;
		va_start(argptr, txt);
        
		vsprintf(buf, txt, argptr);
		fprintf(pInstance->f_descr,"%s",buf);
		//pInstance->log_out << buf;
		va_end(argptr);
	}
}

char * Logger::getSwitchName(int switch_id) {
    
	switch (switch_id) {
            
        case LOG_RECIEVED_DATA:		return "RECEIVED_DATA";
        case LOG_SENT_DATA:		return "LOG_SENT_DATA";
        case LOG_PARSE_TEST:		return "LOG_PARSE_TEST";
        case LOG_COMMUNICATION:		return "LOG_COMMUNICATION";
        case LOG_CONFIG:		return "LOG_CONFIG";
        case LOG_THREADS:		return "LOG_THREADS";
        case LOG_TEMP:			return "LOG_TEMP";
        default:			return "UNKNOWN";
	}
}

//text strings

const char *log_recieved = "received: \n%s\n\n";
const char *log_sent = "sent: \n%s\n\n";
const char *log_parse_test = "Parsing data result : %s\n\n";
const char *log_comm_starting = "Starting communication...\n";
const char *log_comm_started = "Communication started\n";
const char *log_comm_start_failed = "Failed to start communication\n";
const char *log_comm_stopping = "Stopping communication...\n";
const char *log_comm_stopped = "Communication stopped\n";
const char *log_comm_stop_failed = "Failed to stop communication\n";
const char *log_config_failed = "Failed to open config file \"%s\"\n";
const char *log_undefined_usethreads = "Undefined UseThreads value, possible values are YES/NO\n";
const char *log_using_config = "Using config: %s\n";
const char *log_comm_tcpsock_create = "TCP socket created.\n";
const char *log_comm_tcpsock_create_fail = "Failed to create TCP socket.\n";
const char *log_comm_tcpsock_close = "TCP socket closed.\n";
const char *log_comm_tcpsock_close_fail = "Failed to close TCP socket.\n";
const char *log_comm_connected = "Connected to server %s:%d.\n";
const char *log_comm_connected2 = "Connected to server %s:%d.\n";
const char *log_comm_connect_fail = "Failed to connect to server  %s:%d.\n";
const char *log_comm_connect_fail2 = "Failed to connect to server  %s:%d.\n";
const char *log_comm_fail_sending = "Failed to send data.\n";
const char *log_comm_sending_error = "Data sending error. The length of sent data does not match.\n";
const char *log_comm_fail_receiving = "Failed to receive data.\n";
const char *log_comm_receiving_error = "Data receiving error. The length of received data does not match.\n";
const char *log_comm_receiving_no_data = "Data receiving : No data available.\n";
const char *log_comm_out_msg_too_long = "Outgoing message is too long: %d, max: %d";
const char *log_comm_received_data_too_long = "Received data is too long, truncating to %d.";
const char *log_comm_error_select = "Error on select(): %s\n";
const char *log_thread_creating = "Creating thread [%d]...\n";
const char *log_thread_creating_error = "ERROR: Failed to create thread [%d], pthread_create() returned %d\n";
const char *log_thread_created = "Thread [%d] created.\n";
const char *log_thread_joining = "Joining thread  [%d]...\n";
const char *log_thread_joining_error = "ERROR: Failed to join thread [%d], pthread_join() returned %d\n";
const char *log_thread_joined = "Thread [%d] joined with status %d.\n";
const char *log_simtime = "time : %f\n";
const char *log_playermodel_gyroscope = "\tgyr: [%f, %f, %f]\n";
const char *log_playermodel_FRP = "\tFRP: L_Coord[%f, %f, %f]\n\t     R_Coord[%f, %f, %f]\n\n"\
"\t     L_Force[%f, %f, %f]\n\t     R_Force[%f, %f, %f]\n";
const char *log_playermodel_ankles = "\tL ankle: [%f, %f]\n\tR ankle: [%f, %f]\n";
const char *log_playermodel_hips = "\tL hip: [%f, %f]\n\tR hip: [%f, %f]\n";
const char *log_playermodel_shoulders = "\tL shoulder: [%f, %f]\n\tR shoulder: [%f, %f]\n";
const char *log_playermodel_knees = "\tL knee: %f \n\tR knee: %f\n";
const char *log_playermodel_legs = "\tL leg: %f \n\tR leg: %f\n";
const char *log_playermodel_elbows = "\tL elbow: %f \n\tR elbow: %f\n";
const char *log_playermodel_arms = "\tL wrist: %f \n\tR wrist: %f\n";
const char *log_worldmodel_ball = "ball: r: %f,  theta: %f,  phi: %f\n";

const char *log_simplestring = "%s";
const char *log_simplestring_with_float = "%s%f\n";
const char *log_simplestring_with_int = "%s%d\n";
const char *log_simplestring_3_float = "%s %f   %f   %f";
