/*! \file ServerMsgParser.h
 <pre>
 <b>File:</b> ServerMsgParser.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> Header file for parser class ApgServerMsgParser.
 </pre>
 */

/********************************************************************
 APG - an ABNF Parser Generator
 Copyright  (C) 2007 Coast to Coast Research, Inc.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see
 <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
 or write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 created: Sun Mar  2 16:22:08 2008
 filename: ServerMsgParser.h
 author: Lowell Thomas
 lowell@coasttocoastresearch.com
 http://www.coasttocoastresearch.com
 
 purpose: Header file for parser class ApgServerMsgParser.
 
 *********************************************************************/
#ifndef ApgServerMsgParser_h
#define ApgServerMsgParser_h

//{{header.pre
#include "Apg.h"
#include "ServerMsgParserOpt.h"
//}}header.pre

class ApgServerMsgParser {
public:
    
    //{{class.pre
    //}}class.pre
    
    //RULE NAME IDs
    enum
    {
        ID_MESSAGE=0,
        ID_MSGELM=1,
        ID_MSGTIME=2,
        ID_TIMESTR=3,
        ID_TIME=4,
        ID_MSGGAMESTATE=5,
        ID_GSMSGEMB=6,
        ID_GSMSGTIME=7,
        ID_GSMSGPLMODE=8,
        ID_GSMSGUNUM=9,
        ID_GSMSGTEAM=10,
        ID_UNUMNR=11,
        ID_TEAMLEFT=12,
        ID_TEAMRIGHT=13,
        ID_GSSTR=14,
        ID_GSTIME=15,
        ID_PLAYMODE=16,
        ID_MSGGYRORATE=17,
        ID_GYRSTR=18,
        ID_GYRX=19,
        ID_GYRY=20,
        ID_GYRZ=21,
        ID_MSGSEE=22,
        ID_SEESTR=23,
        ID_SEEONEOBJECT=24,
        ID_OBJECTNAME=25,
        ID_SEEX=26,
        ID_SEEY=27,
        ID_SEEZ=28,
        ID_MSGUNIVERSALJOINT=29,
        ID_UJSTR=30,
        ID_UJNAME=31,
        ID_UJAX1=32,
        ID_UJAX2=33,
        ID_MSGHINGE=34,
        ID_HJSTR=35,
        ID_HJNAME=36,
        ID_HJAX=37,
        ID_MSGFORCERESISTANCE=38,
        ID_FRPSTR=39,
        ID_FNAME=40,
        ID_FNAMELF=41,
        ID_FNAMERF=42,
        ID_FRPCX=43,
        ID_FRPCY=44,
        ID_FRPCZ=45,
        ID_FRPFX=46,
        ID_FRPFY=47,
        ID_FRPFZ=48,
        ID_MSGUNKNOW=49,
        ID_MSGTAIL=50,
        ID_SP=51,
        ID_ALPHA=52,
        ID_DIGIT=53,
        ID_REALNR=54,
        ID_ALPHANUMSTRING=55,
        ID_BRACKET_STRING=56,
        ID_STRING_WITHOUT_BR=57,
        ID_STRING=58,
        ID_IDENTIFIER=59,
        ID_COUNT=60
    } RULENAMEIDS;
    
public:
    
    // CLASS FUNCTIONS
    ApgServerMsgParser();
    ~ApgServerMsgParser();
    ulong ulSyntaxAnalysis(ulong ulStartRule, uchar* ucpSrc, ulong ulSrcLen, void* vpData = NULL, ulong ulTrace = APG_FALSE);
    ulong ulSemanticAnalysis(void* vpData);
    
    // CALL BACK FUNCTIONS
    static ulong pfn_Time(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_UnumNr(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_TeamLeft(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_TeamRight(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_GSTime(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_PlayMode(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_GYRx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_GYRy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_GYRz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_ObjectName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_SEEx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_SEEy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_SEEz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_UJName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_UJax1(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_UJax2(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_HJName(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_HJax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FNameLF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FNameRF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPCx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPCy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPCz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPFx(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPFy(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    static ulong pfn_FRPFz(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
    
private:
    static void* m_vpParserInit;
    APG_BSTR m_sSrc;
    
public:
    void* m_vpParserCtx;
    void* m_vpMemCtx;
    APG_PARSER_STATE m_sState;
    APG_PARSER_STATS m_sStats;
    
    //{{class.post
private:
    ServerMsgParserData StoredData;
public:
    int ParseData(uchar* ucpSrc, ulong ulSrcLen);	//function for parsing data
    ServerMsgParserData *GetStoredData(){
        return &StoredData;
    }
    //}}class.post
};

//{{header.post
//}}header.post

#endif // ApgServerMsgParser_h
