/*! \file Joint.cpp
 <pre>
 <b>File:</b> Joint.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/16/2008
 <b>Contents:</b> This class is abstract - it is parent of classes UniversalJoint and HingeJoint.
 </pre>
 */

#include "Joint.h"
#include <limits>

Joint::Joint(char * _name) {
    
	name			= _name;
	simTime			= 0.0;
	deltaSimTime	= 0.0;
	method			= APPROXIMATING;
	startTime		= 0.0;
	timeout			= std::numeric_limits<double>::max();
	lastRecalc		= -1.0;
}

Joint::~Joint() {
}

void Joint::update(double _simTime) {
    
	deltaSimTime	= _simTime - simTime;
	simTime			= _simTime;
}

void Joint::recalculate() {
    
	switch (method) {
            
		case CONSTANT:
			recalculateConstant();
			break;
			
		case APPROXIMATING:
			recalculateApproximating();
			break;
	}
    
	lastRecalc = simTime;
}

bool Joint::recalculated() {
    
	return lastRecalc == simTime;
}

