/*! \file Joint.h
 <pre>
 <b>File:</b> Joint.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/16/2008
 <b>Contents:</b> This class is abstract - it is parent of classes UniversalJoint and HingeJoint.
 </pre>
 */

#ifndef __JOINT_H__
#define __JOINT_H__

#include "Macros.h"
#include <iostream>

#define SIMULATION_STEP						0.02			/*!< time of simulation step in seconds */
#define JOINT_ANGLE_ERROR_CONSTANT			DEG2RAD(1.0)	/*!< joint angle error for constant method */
#define JOINT_ANGLE_ERROR_APPROXIMATING		DEG2RAD(1.0)	/*!< joint angle error for approximation method */
#define CHECK_JOINT_STOPS					false			/*!< true -> stops for joint will be checked for moves */

using namespace std;

enum JointMoveMethod {
    
	CONSTANT = 0,
	APPROXIMATING
};

class Joint {
    
protected:
    
	char * name;
    
	double simTime;											
	double deltaSimTime;
	
	JointMoveMethod method;
    
	double startTime;			/*!< start time of the current move */
	double timeout;				/*!< maximum joint move time */
	double lastRecalc;			/*!< time of last recalculation */
	
	virtual void recalculateConstant() = 0;
	virtual void recalculateApproximating() = 0;
    
public:
	
	Joint(char * _name);
	~Joint();
	
	void update(double _simTime);
	virtual void buildMessage(char * message) = 0;
	virtual bool isMoving() = 0;
    
	void recalculate();
	bool recalculated();
};

#endif /*__JOINT_H__*/

