/*! \file CoordCartesian.cpp
 <pre>
 <b>File:</b> CoordCartesian.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/11/2008
 <b>Contents:</b> This class represents cartesian coordinations and provides useful conversion methods.
 </pre>
 */

#include "CoordCartesian.h"

void CoordCartesian::fromSpherical(double r, double t, double p) {
    
    double sinp = sin(p);
    
    x = r * sinp * cos(t);
    y = r * sinp * sin(t);
    z = r * cos(p);
}

ostream & operator <<(ostream & outStream, const CoordCartesian & cartesian) {
    
    return outStream << cartesian.x << ", " << cartesian.y << ", " << cartesian.z;
}

CoordCartesian::CoordCartesian() {
    
    x = y = z = 0.0;
}

CoordCartesian::CoordCartesian(double _x, double _y, double _z) {
    
    x = _x;
    y = _y;
    z = _z;
}

CoordCartesian::CoordCartesian(const CoordSpherical & spherical) {
    
    fromSpherical(spherical.r, spherical.theta, spherical.phi);
}

CoordCartesian::~CoordCartesian() {
}

CoordCartesian & CoordCartesian::operator =(const CoordCartesian & cartesian) {
    
    x = cartesian.x;
    y = cartesian.y;
    z = cartesian.z;
    
    return *this;
}

CoordCartesian & CoordCartesian::operator =(const CoordSpherical & spherical) {
    
    fromSpherical(spherical.r, spherical.theta, spherical.phi);
    return *this;
}

CoordCartesian CoordCartesian::operator +(CoordCartesian & cartesian) {
    
    return CoordCartesian(x + cartesian.x, y + cartesian.y, z + cartesian.z);
}

CoordCartesian & CoordCartesian::operator +=(CoordCartesian & cartesian) {
    
    x += cartesian.x;
    y += cartesian.y;
    z += cartesian.z;
    
    return *this;
}

CoordCartesian CoordCartesian::operator -(CoordCartesian & cartesian) {
    
    return CoordCartesian(x - cartesian.x, y - cartesian.y, z - cartesian.z);
}

CoordCartesian & CoordCartesian::operator -=(CoordCartesian & cartesian) {
    
    x -= cartesian.x;
    y -= cartesian.y;
    z -= cartesian.z;
    
    return *this;
}
